# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import GDK
import soundcard
import parser
import engravers
import const
import musicalpitch
import gnome.ui

class MusicDisplayer(gtk.GtkScrolledWindow):
    def __init__(self, callback):
        gtk.GtkScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
        self.m_callback = callback
        self.m_engravers = [[],]
        self.m_fontsize = 20
        self.m_clickables = []
        self.g_d = gtk.GtkDrawingArea()
        self.g_d.show()

        self.add_with_viewport(self.g_d)
        self.g_d.connect("expose_event", self.on_expose_event)
        self.g_d.add_events(GDK.EXPOSURE_MASK | GDK.BUTTON_PRESS_MASK)
        self.g_d.connect("button_press_event", self.on_button_press_event)

        self.g_pixmap = None
        self.m_width = self.m_height = 0
    def clear(self, numstaff=1):
        self.m_engravers = [[],]*numstaff
        self.g_d.queue_draw()
        self.g_pixmap = None
        dim = engravers.dimentions[20]
        self.set_usize(-1, numstaff*dim.staff_spacing+dim.first_staff_ypos)
    def display_range(self, music, fontsize, start, end):
        #FIXME handle exceptions
        #FIXME this function only handles music where start == Rat(0, 1) correctly
        if start != soundcard.rat.Rat(0, 1):
            print "warning: musicdisplayer.display_range: start != Rat(0, 1)"
        score = parser.parse_to_score_object(music)
        tv = score.m_timeposdict.keys()
        v = filter(lambda t, end=end: t < end, tv)
        self.m_fontsize = fontsize
        v.sort()
        self.m_engravers = score._generate_engravers(v, fontsize)
        self._display()
    def display(self, music, fontsize, what_part=const.ALL):
        try:
            score = parser.parse_to_score_object(music)
        except musicalpitch.InvalidNotenameException, n:
            gnome.ui.GnomeErrorDialog("Invalid notename: %s" % n.m_notename)
            self.clear()
            return
        if what_part == const.ALL:
           self.m_engravers = score.get_engravers(fontsize)
        elif what_part == const.FIRST:
           self.m_engravers = score.get_first_engravers(fontsize)

        self.m_fontsize = fontsize
        self._display()
    def _display(self):
        dim = engravers.dimentions[self.m_fontsize]
        self.m_width = self.m_engravers[0][-1].m_xpos + 20
        self.m_height = len(self.m_engravers)*dim.staff_spacing+dim.first_staff_ypos
        self.set_usize(-1, self.m_height)
        self.g_d.size(self.m_width, self.m_height-4)
        self.g_pixmap = None
        self.g_d.queue_draw()
    def add_clickable_region(self, x, y, w, h, midi_int):
        self.m_clickables.append({'x':x, 'y':y, 'w':w, 'h':h, 
                                  'midi_int': midi_int})
    def on_button_press_event(self, arg1, event):
        for r in self.m_clickables:
            if r['x'] < event.x < r['x'] + r['w'] \
               and r['y'] < event.y < r['y'] + r['h']:
                self.m_callback(r['midi_int'])
                #soundcard.play_note(0, 4, 0, r['midi_int'])
    def on_expose_event(self, darea, event):
        self.white_gc = self.g_d.get_style().white_gc
        self.black_gc = self.g_d.get_style().black_gc
        dim = engravers.dimentions[self.m_fontsize]
        if self.g_pixmap == None:
            d = self.get_allocation()
            if self.m_width < d[2]:
                self.m_width = d[2]
            if self.m_height < d[3]:
                self.m_height = d[3]
            self.g_pixmap = pixmap = gtk.create_pixmap(self.g_d,
                                     self.m_width, self.m_height)
            gtk.draw_rectangle(pixmap, self.white_gc, gtk.TRUE, 0, 0,
                               self.m_width, self.m_height)
            staff_centrum = dim.first_staff_ypos
            self.m_clickables = []
            for staff in self.m_engravers:
                for e in staff:
                    if isinstance(e, engravers.StemEngraver):
                        continue
                    e.engrave(self.g_d, pixmap, self.black_gc, staff_centrum)
                    if isinstance(e, engravers.NoteheadEngraver):
                        self.add_clickable_region(e.m_xpos + e.m_shift * dim.xshift,
                                    e.m_ypos*dim.linespacing/2+staff_centrum-4,
                                    10, 10, e.m_midi_int)
                # stems has to be drawn after noteheads and accidentals
                for e in staff:
                    if isinstance(e, engravers.StemEngraver):
                        e.engrave(self.g_d, pixmap, self.black_gc, staff_centrum)
                for y in range(-2, 3):
                   gtk.draw_line(pixmap, self.black_gc,
                                 0, staff_centrum+dim.linespacing*y, 
                                 self.m_width, staff_centrum+y*dim.linespacing)
                staff_centrum = staff_centrum + dim.staff_spacing
        darea.draw_pixmap(self.white_gc, self.g_pixmap, 0, 0, 0, 0, self.m_width, self.m_height)



#!/usr/bin/env python

# unfinished script that create a lilypond-book document
# over all the lessonfiles.

import sys, re, string, os, os.path
sys.path.append(".")

import src, src.dataparser, src.lessonfile

def so2ly(s, musicformat):
    re_staff = re.compile("\\\\staff")
    re_addvoice = re.compile("\\\\addvoice")
    v = re_staff.split(s)
    v = filter(lambda i: i!="", v)
    v = map(lambda s, r=re_addvoice: r.split(s), v)
    retval = "\\score{\n  <"
    i = ''
    for s in v:
        i = i + 'i'
        retval = retval + "\n    \\context Staff=%s<" % i
        for vo in s:
            i = i + 'i'
            if musicformat == 'chord':
                m = "< %s >" % vo
            else:
                m = vo
            retval = retval + "\n      \\context Voice=%s\\notes{%s}" % (i, m)
        retval = retval + "\n    >"
    retval = retval + "\n  >\n}"
    return retval

predef = {'dictation': 'dictation',
                      'progression': 'progression',
                      'harmony': 'harmony',
                      'sing-chord': 'sing-chord',
                      'chord': 'chord',
                      'id-by-name': 'id-by-name',
                      'satb': 'satb',
                      'horiz': 'horiz',
                      'vertic': 'vertic',
                      'yes': 1,
                      'no': 0,
                      'tempo': (60, 4)}

parser = src.dataparser.Dataparser(predef, ('tempo',))
parser.parse_file(sys.argv[1])
questions = map(lambda e: e[1],
                               filter(lambda e: e[0] == 'question',
                                      parser.m_blocks[1:]))
H = map(lambda e: e[1], filter(lambda e: e[0] == 'header',
                                       parser.m_blocks))
header = src.lessonfile._Header(H[0])


of = open(os.path.basename(sys.argv[1])+".tex", "w")
of.write(r"""\documentclass[a4paper, 12pt]{article}
\begin{document}
""")
of.write("\n\\begin{itemize}")
of.write("\n\\item title: %s" % header.title)
of.write("\n\\item description: %s" % header.description)
of.write("\n\\item content: %s" % header.content)
of.write("\n\\item musicformat: %s" % header.musicformat)
of.write("\n\\end{itemize}")

for q in questions:
    of.write("""\n\n\\section{Question: %s}
\\begin{lilypond}
%s
\\end{lilypond}
""" % (q['name'], so2ly(q['music'], header.musicformat)))
of.write("\n\\end{document}")
of.close()




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome, gu
from htmlwidget import HtmlWidget
from i18n import _

class SelectLessonfileWidget(gtk.GtkFrame):
    """
    The class will parse call the teacher to parse the
    lessonfile because the widget need some data from the
    lessonfile header.
    """
    def __init__(self, teacher, win, change_cb):
        gtk.GtkFrame.__init__(self, _("Lesson file"))
        self.m_t = teacher
        self.change_cb = change_cb
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        vbox.set_spacing(gnome.uiconsts.PAD)
        self.add(vbox)
        # the box the combos go into
        hbox = gu.bHBox(vbox, gtk.FALSE)

        self.g_collection_combo = gtk.GtkCombo()
        self.g_collection_combo.entry.set_editable(gtk.FALSE)
        hbox.pack_start(self.g_collection_combo)

        self.g_file_combo = gtk.GtkCombo()
        self.g_file_combo.entry.set_editable(gtk.FALSE)
        hbox.pack_start(self.g_file_combo)

        # descriptions
        hbox = gu.bHBox(vbox, gtk.FALSE)
        gu.bLabel(hbox, _("Title")+": ", gtk.FALSE)
        self.g_title = gu.bLabel(hbox, "", gtk.FALSE)

        #details
        self.g_toggle_description = gtk.GtkToggleButton(_("Details"))
        self.g_toggle_description.connect('clicked', self.toggle_lessonfile_details)
        hbox.pack_end(self.g_toggle_description, gtk.FALSE, gtk.FALSE)
        self.g_description_frame = gtk.GtkFrame()
        self.g_lessonfile_details = HtmlWidget(self.m_t.m_app.handle_href, None)
        self.g_description_frame.add(self.g_lessonfile_details)
        vbox.pack_start(self.g_description_frame)
        self.m_lock = 0
        self.update()
        self.g_collection_combo.entry.connect('changed', self.change_collection)
        self.g_file_combo.entry.connect('changed', self.change_file)
        self._fw = self.m_t.add_watch('lessonfile', self._lessonfilewatch)
        self._cw = self.m_t.add_watch('lessoncollection', self._lessoncollectionwatch)
    def _lessonfilewatch(self, s):
        self.m_t.remove_watch('lessonfile', self._fw)
        self._fw = self.m_t.add_watch('lessonfile', self._lessonfilewatch)
    def _lessoncollectionwatch(self, s):
        self.m_t.remove_watch('lessoncollection', self._cw)
        self.m_lock = self.m_lock + 1
        self.g_collection_combo.entry.set_text(self.m_t.get_string(s))
        self.m_lock = self.m_lock - 1
        self._cw = self.m_t.add_watch('lessoncollection', self._lessoncollectionwatch)
    def change_collection(self, widget):
        self.m_t.set_lessoncollection(widget.get_text())
        self.update(1)
        self.change_cb()
    def change_file(self, widget):
        if self.m_lock > 0:
            return
        self.m_t.set_lessonfile(widget.get_text())
        self.update(2)
        self.change_cb()
    def update(self, level=0):
        """0: everything
           1: don't include collection combo
           2: only information label
        """
        if self.m_lock > 0:
            return
        self.m_lock = self.m_lock + 1
        if level == 0:
            self.g_collection_combo.set_popdown_strings(
                    self.m_t.m_app.lessonfile_manager.get_collections())
            self.g_collection_combo.entry.set_text(
                    self.m_t.get_string('lessoncollection'))
        if level <= 1:
            F = self.m_t.m_app.lessonfile_manager.get_filenames( \
                    self.m_t.get_string('lessoncollection'), self.m_t.m_content_type)
            if F:
                self.g_file_combo.set_popdown_strings(F)
                self.g_file_combo.entry.set_text(self.m_t.get_string('lessonfile'))
                self.g_file_combo.set_sensitive(gtk.TRUE)
            else:
                self.g_file_combo.entry.set_text('')
                self.g_file_combo.set_sensitive(gtk.FALSE)
                self.g_title.set_text('')
                self.g_description_frame.hide()
                #self.g_lessonfile_details.source("<html></html>")
                self.g_toggle_description.set_active(gtk.FALSE)
                self.g_toggle_description.set_sensitive(gtk.FALSE)
        if not self.m_t.m_P:
            self.m_lock = self.m_lock - 1
            return
        # the info labels
        if self.m_t.m_P:
            self.g_title.set_text(self.m_t.m_P.header.title)
        else:
            self.g_title.set_text('')
        s = self.m_t.m_P.header.description
        if s:
            self.g_lessonfile_details.source("<html><body bgcolor=white>%s</body></html>" % s)
            self.g_toggle_description.set_sensitive(gtk.TRUE)
        else:
           self.g_lessonfile_details.source("<html></html>")
           self.g_toggle_description.set_active(gtk.FALSE)
           self.g_toggle_description.set_sensitive(gtk.FALSE)
        self.m_lock = self.m_lock - 1
    def toggle_lessonfile_details(self, widget=None):
        if widget.active:
            self.g_description_frame.show()
        else:
            self.g_description_frame.hide()
    def show(self):
        self.show_all()
        self.g_description_frame.hide()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, string, re
import app, mpd

import src.rcfile
src.rcfile.copy_rc14_file()

try:
    import readline
except:
    print "Failed to import readline"

del sys.argv[sys.argv.index('--txtui')]
import harmonicintervall
import configureoutput

whitespace_re = re.compile("\s+")

class UI:
    def __init__(self, argv):
        self.m_prompt = "solfege>"
        self.m_app = app.SolfegeApp(argv, self)
        self.mainloop()
    def display_docfile(self, fn):
        print "display file (not finished yet!):" , fn
    def display_error_message(self, txt):
        print "Error message:\n", txt
    def mainloop(self):
        print "Welcome to GNU Solfege %s\nType 'help' to list commands." % configureoutput.VERSION_STRING
        teacher = None
        while 1:
            s = raw_input(self.m_prompt)
            s = string.rstrip(s)
            v = whitespace_re.split(s)
            if s == '':
                print "Type 'help' to list commands."
                continue
            if s in ('exit', 'bye', 'quit'):
                break
            if s == 'help':
                print """
start harm            start the harmonic exercise
new                   new question
repeat                repeat question
repeat melodic        repeat melodic
guess xxx             xxx is short name of intervall, for example
                      m2 for minor second, M3 for major third,
                      4 for perfect fourth, d5 for diminished fifth.
exit                  quit the program.
"""
            elif v[0] == 'start':
                if len(v) < 2:
                    print "You must tell what exercise to start."
                    print "This release only support 'start harm' that will start the"
                    print "harmonic intervall exercise."
                    continue
                if v[1] == 'harm' and teacher == None:
                    self.m_prompt = "harmonic-intervall>"
                    teacher = harmonicintervall.Teacher('harmonic-intervall', self.m_app)
            elif v[0] == 'new':
                if not teacher:
                    print 'start an exercise first'
                    continue
                teacher.new_question()
                teacher.play_question()
            elif v[0] == 'repeat':
                if len(v) == 1:
                    teacher.play_question()
                elif v[1] == 'melodic':
                    teacher.play_melodic()
            elif v[0] == 'guess':
                if len(v) < 2:
                    print "missing argument"
                    continue
                i = mpd.Intervall()
                i.set_from_string(v[1])
                if teacher.guess_answer(i.get_intvalue()):
                    print "Correct"
                else:
                    print "Wrong"
            else:
                print "sorry, I don't understand you."

w = UI(sys.argv)



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from i18n import _, _i

import gtk
import GDK

SHORT_WAIT = 700
NORMAL_WAIT = 2000

USE_CHORD = 'chord'
USE_HARMONY = 'harmony'
USE_ID_BY_NAME = 'id-by-name'
USE_DICTATION = 'dictation'
USE_SING_CHORD = 'sing-chord'

# These are the exercises that are available as icons on the toolbar
# and listed on the _File menu. 
# The list should have another name. Also, if the number of exercises
# increase, it should be configurable what exercises are available
# from the toolbar.
exercises = ('chord',
             'harmonic-intervall',
             'melodic-intervall',
             'sing-intervall',
             'identify-scale',
             'id-by-name',
             'dictation',
             'twelve-tone',
             'id-tone',
             'compare-intervalls',
             'sing-chord',
             'rhythm')

exercisedata = {
   # id: prettyname, menulabel, pixmap
    'chord': (_("_Chords"), 'chord24.xpm', GDK.F2, 0),
    'harmonic-intervall': (_("_Harmonic intervalls"),
        'harmonicintervall24.xpm', GDK.F3, 0),
    'melodic-intervall': (_("_Melodic intervalls"),
        'melodicintervall24.xpm', GDK.F4, 0),
    'sing-intervall': (_("_Sing intervall"), 'singintervall24.xpm', GDK.F5, 0),
    'identify-scale': (_("_Identify scale"), 'identifyscale24.xpm', GDK.F6, 0),
    'id-by-name': (_("Identify by _name"), 'idbyname24.xpm', GDK.F7, 0),
    'dictation': (_("_Dictation"), 'dictation24.xpm', GDK.F8, 0),
    'harmonic-progression-dictation': (_("Harmonic _progression dictation"),
        'harmonicprogression24.xpm', GDK.F9, 0),
    'twelve-tone': (_("Sing _twelvetone"), 'twelvetone24.xpm', GDK.F10, 0),
    'id-tone': (_("Id_entify tone"), 'idtone24.xpm', GDK.F2, GDK.CONTROL_MASK),
    'compare-intervalls': (_("C_ompare intervalls"),
        'compareintervalls24.xpm', GDK.F3, GDK.CONTROL_MASK),
    'sing-chord': (_("Sin_g chord"),
        'singchord24.xpm', GDK.F4, GDK.CONTROL_MASK),
    'rhythm': (_("Rh_ythm"), 'rhythm24.xpm', GDK.F5, GDK.CONTROL_MASK),
    'polyrhythm': ("_Polyrhythmic", "", GDK.F6, GDK.CONTROL_MASK),
    'identify-bpm': ("Bpm", "", None, None),
    'intervalls': ("Intervall", "", None, None),
    'tuner': ("Tuner", "", None, None),
}


QSTATUS_NO = 0
QSTATUS_NEW = 1
QSTATUS_WRONG = 2
QSTATUS_SOLVED = 3
QSTATUS_GIVE_UP = 4

int_intervall = (_("unison"), _("minor second"), _("major second"),
                 _("minor third"), _("major third"), _("fourth"),
                 _("diminished fifth"), _("perfect fifth"),
                 _("minor sixth"), _("major sixth"),
                 _("minor seventh"), _("major seventh"),
                 _("perfect octave"),
                 _("minor ninth"), _("major ninth"),
                 _("minor decim"), _("major decim"))

short_intervall_name = (_i("u"), _i("m2"), _i("M2"), _i("m3"), _i("M3"), _i("4"),
                        _i("d5"), _i("5"), _i("m6"), _i("M6"), _i("m7"),
                        _i("M7"), _i("8"), _i("m9"), _i("M9"), _i("m10"),
                        _i("M10"))




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path, shutil, __main__
import src.configureoutput

def copy_rc14_file():
    """
    This function has to be called before we import gnome.config
    """
    __main__.rc_copied = 0
    if src.configureoutput.WITH_GNOME:
        rc10 = os.path.expanduser("~/.gnome/solfege1.0")
        rc11 = os.path.expanduser("~/.gnome/solfege1.1")
        rc12 = os.path.expanduser("~/.gnome/solfege1.2")
    else:
        rc10 = os.path.expanduser("~/.solfegerc1.0")
        rc11 = os.path.expanduser("~/.solfegerc1.1")
        rc12 = os.path.expanduser("~/.solfegerc1.2")
    rc13 = os.path.expanduser("~/.solfegerc1.3")
    rc14 = os.path.expanduser("~/.solfegerc1.4")

    if not os.path.isfile(rc14):
        if os.path.isfile(rc13):
            shutil.copy(rc13, rc14)
            __main__.rc_copied = 1
        elif os.path.isfile(rc12):
            shutil.copy(rc12, rc14)
            __main__.rc_copied = 1
        elif os.path.isfile(rc11):
            shutil.copy(rc11, rc14)
            __main__.rc_copied = 1
        elif os.path.isfile(rc10):
            shutil.copy(rc10, rc14)
            __main__.rc_copied = 1





# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import gu, widgets, inputwidgets
import soundcard, mpd
import abstract, const, utils
from i18n import _
import statistics, statisticsviewer

class Teacher(abstract.Teacher):
    OK = 0
    ERR_PICKY = 1
    ERR_NO_INTERVALLS = 2
    ERR_NOTERANGE = 3
    def __init__(self, exname, app):
        abstract.Teacher.__init__(self, exname, app)
        self.m_statistics = statistics.HarmonicIntervallStatistics(self)
        self.m_tonika = None
        self.m_intervall = None
    def new_question(self):
        """
        Return values:
        OK: new question created, all ok
        ERR_NO_INTERVALLS: no new question because no intervalls are selected
        ERR_PICKY: you have to solve this question before you are allowed to create new
        """
        if self.m_timeout_handle:
            gtk.timeout_remove(self.m_timeout_handle)
            self.m_timeout_handle = None

        if self.get_bool('config/picky_on_new_question') \
                 and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return Teacher.ERR_PICKY

        self.q_status = const.QSTATUS_NO

        if self.get_list('intervalls') == []:
            return Teacher.ERR_NO_INTERVALLS
        last_question = self.m_intervall
        last_tonika = self.m_tonika
        L, H = utils.adjust_low_high_to_irange(self.get_string('lowest_tone'),
           self.get_string('highest_tone'), self.get_list('intervalls'))
        while 1:
            # in this loop we will try to make a question that is not the
            # same that the last one.
            self.m_tonika, self.m_intervall = \
                  utils.random_tonika_and_intervall(L, H,
                    self.get_list('intervalls'))
            if last_question is None:
                break
            if (self.m_intervall == last_question
                and self.m_tonika == last_tonika) \
                and (len(self.get_list('intervalls')) > 1 or (H - L) > 1):
                continue
            break
        assert self.m_tonika
        self.q_status = const.QSTATUS_NEW
        return Teacher.OK
    def guess_answer(self, answer):
        """
        Return: 1 if correct, None if wrong
        """
        assert self.q_status not in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP)
        if self.m_intervall == answer:
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_correct(self.m_intervall)
            self.maybe_auto_new_question()
            self.q_status = const.QSTATUS_SOLVED
            return 1
        else:
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_wrong(self.m_intervall, answer)
                self.q_status = const.QSTATUS_WRONG
    def give_up(self):
        """This function is only called *after* the user already has
        answered wrong once, so the statistics are already updated.
        """
        self.q_status = const.QSTATUS_GIVE_UP
    def play_question(self):
        if self.q_status == const.QSTATUS_NO:
            return
        instr_low, instr_low_vel, instr_high, instr_high_vel = self.get_instrument_config(2)
        low_tone = self.m_tonika.semitone_pitch()
        high_tone = (self.m_tonika+self.m_intervall).semitone_pitch()

        m = soundcard.Track()
        m.set_bpm(self.get_int('config/default_bpm'))
        m.set_patch(0, instr_low)
        m.set_patch(1, instr_high)
        m.start_note(0, low_tone, instr_low_vel)
        m.start_note(1, high_tone, instr_high_vel)
        m.notelen_time(4)
        m.stop_note(0, low_tone, instr_low_vel)
        m.stop_note(1, high_tone, instr_high_vel)
        soundcard.synth.play_track(m)
        return 1
    def play_melodic(self):
        if self.q_status == const.QSTATUS_NO:
            return
        if self.get_bool('override_default_instrument'):
            instr_low = self.get_int('lowest_instrument')
            instr_low_vel = self.get_int('lowest_instrument_velocity')
            instr_high = self.get_int('highest_instrument')
            instr_high_vel = self.get_int('highest_instrument_velocity')
        else:
            instr_low = instr_high = self.get_int('config/preferred_instrument')
            instr_low_vel = instr_high_vel \
                    = self.get_int('config/preferred_instrument_velocity')
        low_tone = self.m_tonika.semitone_pitch()
        high_tone = (self.m_tonika+self.m_intervall).semitone_pitch()
        m = soundcard.Track()
        m.set_bpm(self.get_int('config/default_bpm'))
        m.set_patch(0, instr_low)
        m.set_patch(1, instr_high)
        m.note(4, 0, low_tone, instr_low_vel)
        m.note(4, 1, high_tone, instr_high_vel)
        soundcard.synth.play_track(m)


class Gui(abstract.IntervallGui):
    def __init__(self, teacher, window):
        abstract.IntervallGui.__init__(self, teacher, window)
        ################
        # practice_box #
        ################
        self.g_repeat_melodic = gu.bButton(self.action_area,
            _("Repeat melodic"), lambda _o, self=self: self.m_t.play_melodic())

        self.g_repeat_melodic.set_sensitive(gtk.FALSE)

        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
        self.g_give_up.set_sensitive(gtk.FALSE)
        ##############
        # config_box #        
        ##############
        frame = gtk.GtkFrame(_("Ask for these intervalls"))
        self.config_box.pack_start(frame, gtk.FALSE)

        self.g_intervall_selector = widgets.IntervallCheckBox(
                                          self.m_exname, 'intervalls')

        self.g_intervall_selector.set_border_width(gnome.uiconsts.PAD)
        frame.add(self.g_intervall_selector)
        #------we need some space
        self.config_box.pack_start(gtk.GtkHBox(), gtk.FALSE,
                                   padding=gnome.uiconsts.PAD_SMALL)
        # -------------------------------------------------
        hbox = gu.bHBox(self.config_box, gtk.FALSE)
        hbox.set_spacing(gnome.uiconsts.PAD)
        hbox.pack_start(gtk.GtkLabel(_("Try to keep question with range from")),
                        gtk.FALSE)
        lowspin = widgets.SimplestNotenameSpinButton(self.get_string('lowest_tone'))
        hbox.pack_start(lowspin, gtk.FALSE)
        gu.bLabel(hbox, _("to"), gtk.FALSE)
        highspin = widgets.SimplestNotenameSpinButton(self.get_string('highest_tone'))
        hbox.pack_start(highspin, gtk.FALSE)
        self.m_notenamerange = widgets.nNotenameRangeController(lowspin, highspin,
                           mpd.LOWEST_NOTENAME, mpd.HIGHEST_NOTENAME,
                           self.m_t.m_exname, 'lowest_tone', 'highest_tone')

        # ------------------------------------------
        self._add_auto_new_question_gui(self.config_box)
        # ----------------------------------------------
        hbox = gu.bHBox(self.config_box, gtk.FALSE)
        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
        gu.bLabel(hbox, _("Input interface:"), gtk.FALSE)
        combo = gu.nCombo(self.m_exname, 'inputwidget', _("Buttons"),
                   inputwidgets.inputwidget_names)
        hbox.pack_start(combo, gtk.FALSE)
        combo.entry.connect('changed',
               lambda entry, self=self, inputwidgets=inputwidgets:
               self.use_inputwidget(inputwidgets.name_to_inputwidget(
                           entry.get_text(), self.click_on_intervall)))
        # ------------ frame -------------------
        self.g_instrument_configurator = widgets.InstrumentConfigurator(
                                                             self.m_exname, 2)
        self.config_box.pack_start(self.g_instrument_configurator, gtk.FALSE)
        self.config_box.set_spacing(0)
        self.config_box.show_all()
        ##############
        # statistics #
        ##############
        self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                   _("Harmonic intervall"))
        self.m_key_bindings['repeat_melodic_ak'] = self.m_t.play_melodic
        self.select_inputwidget()
    def give_up(self, _o=None):
        if self.m_t.q_status == const.QSTATUS_WRONG:
            self.g_flashbar.push(_("The answer is: %s")
                 % const.int_intervall[self.m_t.m_intervall])
            self.m_t.give_up()
            self.g_new_intervall.set_sensitive(gtk.TRUE)
            self.g_give_up.set_sensitive(gtk.FALSE)
    def click_on_intervall(self, mouse_button, intervall, midi_int):
        if mouse_button == 1:
            if self.m_t.q_status == const.QSTATUS_NO:
                self.g_flashbar.flash(_("Click 'New intervall' to begin."))
                return
            if not -17 < intervall < 17:
                #FIXME  fix and remove
                self.g_flashbar.flash(_("Ignoring intervalls greater than decim."))
                self.g_input.forget_last_tone()
                return
            if self.m_t.q_status == const.QSTATUS_SOLVED:
                if self.m_t.guess_answer(intervall):
                    self.g_flashbar.flash(_("Correct, but you have already solved this question"))
                else:
                    self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
            elif self.m_t.q_status in (const.QSTATUS_NEW, const.QSTATUS_WRONG):
                if self.m_t.guess_answer(intervall):
                    self.g_flashbar.flash(_("Correct"))
                    self.g_new_intervall.set_sensitive(gtk.TRUE)
                    self.g_give_up.set_sensitive(gtk.FALSE)
                else:
                    self.g_flashbar.flash(_("Wrong"))
                    self.g_give_up.set_sensitive(gtk.TRUE)
                    if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
                        self.m_t.play_question()
            self.g_input.set_first_note(self.m_t.m_tonika)
        elif mouse_button == 2:
            # we only get here if you use one if the instrument widgets, not
            # with the buttons interface.
            soundcard.play_note(self.get_int('config/preferred_instrument'),
                          4, 0, midi_int,
                          self.get_int('config/preferred_instrument_velocity'))
        elif mouse_button == 3 and self.m_t.q_status != const.QSTATUS_NO:
            instr_low, instr_low_vel, instr_high, instr_high_vel = self.m_t.get_instrument_config(2)
            n2 = self.m_t.m_tonika + intervall
            m = soundcard.Track()
            m.set_patch(0, instr_low)
            m.set_patch(1, instr_high)
            m.start_note(0, self.m_t.m_tonika.semitone_pitch(), instr_low_vel)
            m.start_note(1, n2.semitone_pitch(), instr_low_vel)
            m.notelen_time(4)
            m.stop_note(0, self.m_t.m_tonika.semitone_pitch(), instr_high_vel)
            m.stop_note(1, n2.semitone_pitch(), instr_high_vel)
            soundcard.synth.play_track(m)
    def new_question(self, _o=None):
        """This function is called when you click on the 'New intervall'
        button, or when you use the key bindings. So it can be called even
        if the 'New intervall' button is insensitive.
        """
        g = self.m_t.new_question()
        if g == Teacher.OK: # new question, everything is OK
            self.g_repeat.set_sensitive(gtk.TRUE)
            self.g_repeat_melodic.set_sensitive(gtk.TRUE)
            self.g_input.set_first_note(self.m_t.m_tonika)
            self.g_new_intervall.set_sensitive(
              not self.get_bool('config/picky_on_new_question'))
            self.m_t.play_question()
            self.g_flashbar.clear()
        elif g == Teacher.ERR_PICKY:
                self.g_flashbar.flash(_("You have to solve this question first."))
        elif g == Teacher.ERR_NO_INTERVALLS:
            self.g_repeat.set_sensitive(gtk.FALSE)
            self.g_repeat_melodic.set_sensitive(gtk.FALSE)
            self.g_flashbar.clear()
            self.g_flashbar.flash(
                    _("You have to select some intervalls to practise."))
    def on_start_practise(self):
        self.m_t.m_statistics.reset_session()
        gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
            self.g_flashbar.flash(_("Click 'New intervall' to begin.")))
    def on_end_practise(self):
        self.g_new_intervall.set_sensitive(gtk.TRUE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_repeat_melodic.set_sensitive(gtk.FALSE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.g_flashbar.clear()
        self.g_input.clear()
        self.m_t.end_practise()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import random

import gtk, gnome
import abstract, gu, const, mpd, mpd.musicdisplayer
import utils
import statistics, statisticsviewer
from i18n import _

scaledata = {
    'ionian': ('[c8 d][e f][g a][b c]|[c b][a g][f e][d c]',
               _("Ionian"), 'c', 0, 0),
    'dorian': ('[c8 d][es f][g a][bes c]|[c bes][a g][f es][d c]',
               _("Dorian"), 'bes', 1, 0),
    'phrygian': ('[c8 des][es f][g as][bes c]|[c bes][as g][f es][des c]',
                _("Phrygian"), 'as', 2, 0),
    'lydian': ('[c8 d][e fis][g a][b c]|[c b][a g][fis e][d c]',
               _("Lydian"), 'g', 3, 0),
    'mixolydian': ('[c8 d][e f][g a][bes c]|[c bes][a g][f e][d c]',
                   _("Mixolydian"), 'f', 0, 1),
    'aeolian': ('[c8 d][es f][g as][bes c]|[c bes][as g][f es][d c]',
                _("Aeolian"), 'es', 1, 1),
    'locrian': ('[c8 des][es f][ges as][bes c]|[c bes][as ges][f es][des c]',
                _("Locrian"), 'des', 2, 1),
    'harmonic_minor': ('[c8 d][es f][g as][b c]|[c b][as g][f es][d c]',
                       _("Harmonic minor"), 'es', 3, 1),
    'melodic_minor': ('[c8 d][es f][g a][b c]|[c bes][as g][f es][d c]',
                      _("Melodic minor"), 'es', 0, 2)
}
class Teacher(abstract.Teacher):
    OK = 0
    ERR_NO_SCALES = 1
    ERR_PICKY = 2
    def scale_key_to_name(self, key):
        return scaledata[key][1]
    def __init__(self, exname, app):
        abstract.Teacher.__init__(self, exname, app)
        self._scale = None
        self.m_statistics = statistics.Statistics(self)
        self.m_statistics.key_to_pretty_name = self.scale_key_to_name
    def give_up(self):
        self.q_status = const.QSTATUS_GIVE_UP
    def new_scale(self):
        """Return 1 if we have a new question, otherwise None
        """
        if self.m_timeout_handle:
            gtk.timeout_remove(self.m_timeout_handle)
            self.m_timeout_handle = None
        if self.get_bool('config/picky_on_new_question') \
                 and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return Teacher.ERR_PICKY
        self.q_status = const.QSTATUS_NO
        active_scales = []
        for s in scaledata.keys():
            if self.get_bool(s):
                active_scales.append(s)
        if active_scales:
            self._scale = random.choice(active_scales)
            self.m_tonika = mpd.MusicalPitch().randomize("f", "c''")
            self.q_status = const.QSTATUS_NEW
            return self.OK
        else:
            return self.ERR_NO_SCALES
    def get_mpd_string(self):
        assert self._scale
        return r"\staff\transpose %s\relative c'{ \clef violin \key %s \major %s }" \
                   % (self.m_tonika.str(), scaledata[self._scale][2], scaledata[self._scale][0])
    def play(self):
        if self.q_status == const.QSTATUS_NO:
            return
        if self.get_bool('override_global_settings'):
            bpm = self.get_int('default_bpm')
        else:
            bpm = self.get_int('config/default_bpm')
        mpd.play_music(self.get_mpd_string(), bpm,
                       self.get_int('config/preferred_instrument'),
                       self.get_int('config/preferred_instrument_velocity'))
    def play_slowly(self):
        if self.q_status == const.QSTATUS_NO:
           return
        if self.get_bool('override_global_settings'):
            bpm = self.get_int('slow_bpm')
        else:
            bpm = self.get_int('config/slow_bpm')
        mpd.play_music(self.get_mpd_string(), bpm,
                       self.get_int('config/preferred_instrument'),
                       self.get_int('config/preferred_instrument_velocity'))
    def guess_answer(self, answer):
        """
        Return: 1 if correct, None if not
        """
        if answer == self._scale:
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_correct(self._scale)
            self.maybe_auto_new_question()
            self.q_status = const.QSTATUS_SOLVED
            return 1
        else:
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_wrong(self._scale, answer)
                self.q_status = const.QSTATUS_WRONG
    def configure_exercise(self, x, y, dict):
        for k in dict.keys():
            if k == 'scales':
                continue
            self.set_string(k, dict[k])
        if dict.has_key('scales'):
            scales = eval(dict['scales'])
            for s in scaledata.keys():
                self.set_bool(s, s in scales)

class Gui(abstract.Gui):
    def __init__(self, teacher, window):
        abstract.Gui.__init__(self, teacher, window)
        self.m_key_bindings = {
           'new_ak': self.new_question,
           'repeat_ak': self.m_t.play,
           'repeat_slowly_ak': self.m_t.play_slowly,
           'give_up_ak': self.give_up}
        ################
        # practise_box #
        ################
        self.g_score_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.g_score_displayer.set_usize(-1, 150)
        self.practise_box.pack_start(self.g_score_displayer)

        self.g_flashbar = gu.FlashBar()
        self.g_flashbar.show()
        self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
        self.practise_box.set_spacing(gnome.uiconsts.PAD)

        self.table = gtk.GtkTable(3, 3, gtk.TRUE)
        self.practise_box.pack_start(self.table, gtk.FALSE)
        self.g_scale_checkbuttons = {}
        for k in scaledata.keys():
            b = gtk.GtkButton(scaledata[k][1])
            b.connect('clicked', self.on_scale_click, k)
            self.table.attach(b, scaledata[k][3], scaledata[k][3]+1,
                              scaledata[k][4], scaledata[k][4]+1)
        self.g_new = gu.bButton(self.action_area, _("New scale"),
                                   self.new_question)
        self.g_repeat = gu.bButton(self.action_area, _("Repeat scale"),
                 lambda _o, self=self: self.m_t.play())
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_repeat_slowly = gu.bButton(self.action_area, _("Repeat slowly"),
                 lambda _o, self=self: self.m_t.play_slowly())
        self.g_repeat_slowly.set_sensitive(gtk.FALSE)
        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.practise_box.set_spacing(gnome.uiconsts.PAD)
        self.practise_box.show_all()
        ##############
        # config_box #
        ##############
        self.config_box.set_spacing(gnome.uiconsts.PAD)
        frame = gtk.GtkFrame(_("Ask for these scales:"))
        self.config_box.pack_start(frame, gtk.FALSE)
        table = gtk.GtkTable(4, 3)
        table.set_border_width(gnome.uiconsts.PAD_SMALL)
        frame.add(table)
        for k in scaledata.keys():
            b = gu.nCheckButton(self.m_exname, k, scaledata[k][1], default_value=1)
            table.attach(b, scaledata[k][3], scaledata[k][3]+1,
                         scaledata[k][4], scaledata[k][4]+1)
        frame = gtk.GtkFrame(_("Override global settings"))
        self.config_box.pack_start(frame, gtk.FALSE)
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        frame.add(vbox)
        self.g_override_global_settings = \
               gu.nCheckButton(self.m_exname, 'override_global_settings',
                            _("Enable"), callback=self.update_config_override)
        vbox.pack_start(self.g_override_global_settings, gtk.FALSE)
        self.mt = table = gtk.GtkTable()
        table.set_col_spacings(50)
        vbox.pack_start(table, gtk.FALSE)
        gu.tLabel(table, 0, 1, 0, 1, _("Slow bpm:"),
                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
        gu.tLabel(table, 0, 1, 1, 2, _("Default bpm:"),
                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
        table.attach(gu.nSpinButton(self.m_exname, 'slow_bpm',
                               gtk.GtkAdjustment(0, 10, 255, 1, 10), digits=0),
                     1, 2, 0, 1, xoptions=0)
        table.attach(gu.nSpinButton(self.m_exname, 'default_bpm',
                               gtk.GtkAdjustment(0, 10, 255, 1, 10), digits=0),
                     1, 2, 1, 2, xoptions=0)
        self.update_config_override()
        # ------------------------------------------
        self._add_auto_new_question_gui(self.config_box)
        # ----------------------------------------------

        self.config_box.show_all()
        ###############
        # statistics
        ###############
        self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                   _("Identify scale"))
    def update_config_override(self, _o=None):
        if self.g_override_global_settings.active:
            self.mt.set_sensitive(gtk.TRUE)
        else:
            self.mt.set_sensitive(gtk.FALSE)
    def give_up(self, widget=None):
        if self.m_t.q_status == const.QSTATUS_WRONG:
            fontsize = self.get_int('config/feta_font_size=20')
            self.g_score_displayer.display(self.m_t.get_mpd_string(), fontsize)
            self.m_t.give_up()
            self.g_new.set_sensitive(gtk.TRUE)
            self.g_give_up.set_sensitive(gtk.FALSE)
            self.g_flashbar.push(_("The answer is: %s") % self.m_t.scale_key_to_name(self.m_t._scale))
    def new_question(self, widget=None):
        self.g_score_displayer.clear()
        g = self.m_t.new_scale()
        if g == Teacher.OK:
            self.m_t.play()
            self.g_repeat_slowly.set_sensitive(gtk.TRUE)
            self.g_repeat.set_sensitive(gtk.TRUE)
            self.g_give_up.set_sensitive(gtk.FALSE)
            self.g_new.set_sensitive(
                     not self.get_bool('config/picky_on_new_question'))
            self.g_flashbar.clear()
        elif g == Teacher.ERR_NO_SCALES:
            self.g_flashbar.flash(_("You have to select some scales to practise."))
            self.g_give_up.set_sensitive(gtk.FALSE)
            self.g_repeat_slowly.set_sensitive(gtk.FALSE)
            self.g_repeat.set_sensitive(gtk.FALSE)
            self.g_flashbar.clear()
        elif g == Teacher.ERR_PICKY:
            self.g_flashbar.flash(_("You have to solve this question first."))
        else:
            raise "NEVER REACH THIS!"
    def on_start_practise(self):
        self.m_t.m_statistics.reset_session()
        self.g_score_displayer.clear()
        gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
           self.g_flashbar.flash(_("Click 'New scale' to begin.")))
    def on_end_practise(self):
        self.g_new.set_sensitive(gtk.TRUE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_repeat_slowly.set_sensitive(gtk.FALSE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.g_flashbar.clear()
        self.m_t.end_practise()
    def on_scale_click(self, button, name):
        if self.m_t.q_status == const.QSTATUS_GIVE_UP:
            return
        if self.m_t.q_status == const.QSTATUS_NO:
            self.g_flashbar.flash(_("Click 'New scale' to begin."))
            return
        if self.m_t.q_status == const.QSTATUS_SOLVED:
            if self.m_t.guess_answer(name):
                self.g_flashbar.flash(_("Correct, but you have already solved this question"))
            else:
                self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
        else:
            if self.m_t.guess_answer(name):
                self.g_flashbar.flash(_("Correct"))
                self.g_new.set_sensitive(gtk.TRUE)
                self.g_give_up.set_sensitive(gtk.FALSE)
            else:
                self.g_flashbar.flash(_("Wrong"))
                if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
                    self.m_t.play()
                self.g_give_up.set_sensitive(gtk.TRUE)




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import gu, widgets
import abstract, const, mpd
import statistics, statisticsviewer
import lessonfile
import string
from i18n import _


class Teacher(abstract.LessonbasedTeacher):
    OK = 0
    ERR_PICKY = 1
    ERR_NO_LESSONFILE = 2
    def __init__(self, exname,  app, use=const.USE_ID_BY_NAME):
        abstract.LessonbasedTeacher.__init__(self, exname, app, use)
        self.lessonfileclass = lessonfile.IdByNameLessonfile
        self.parse_lessonfile()

        self.m_statistics = statistics.LessonStatistics(self)
    def give_up(self):
        self.q_status = const.QSTATUS_GIVE_UP
    def new_question(self):
        if self.m_timeout_handle:
            gtk.timeout_remove(self.m_timeout_handle)
            self.m_timeout_handle = None
 
        if self.get_bool('config/picky_on_new_question') \
                 and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return Teacher.ERR_PICKY

        self.q_status = const.QSTATUS_NO

        if self.m_P:
            self.m_P.select_random_question()
            self.q_status = const.QSTATUS_NEW
            return self.OK
        else:
            return self.ERR_NO_LESSONFILE
    def guess_answer(self, answer):
        """
        Return: 1 if correct, None if wrong
        """        
        if answer == self.m_P.get_untranslated_name():
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_correct(answer)
            self.maybe_auto_new_question()
            self.q_status = const.QSTATUS_SOLVED
            return 1
        else:
            if self.q_status == const.QSTATUS_NEW:
                self.m_statistics.add_wrong(self.m_P.get_untranslated_name(),
                                            answer)
                self.q_status = const.QSTATUS_WRONG
        

class Gui(abstract.LessonbasedGui):
    def __init__(self, teacher, window):
        abstract.LessonbasedGui.__init__(self, teacher, window)
        self.m_key_bindings = {'new_ak': self.new_question,
                               'repeat_ak': self.m_t.play_question,
                               'give_up_ak': self.give_up,
                               'show_ak': self.show_answer}
        ################
        # practise_box #
        ################
        self.g_bb = widgets.QuestionNameButtonTable()
        self.practise_box.pack_start(self.g_bb, gtk.FALSE)
        
        self.g_flashbar = gu.FlashBar()
        self.g_flashbar.show()
        self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
        self.practise_box.set_spacing(gnome.uiconsts.PAD)

        self.g_new = gu.bButton(self.action_area, _("New"), self.new_question)
        self.g_repeat = gu.bButton(self.action_area, _("Repeat"),
                                   lambda _o, self=self: self.m_t.play_question())
        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
        self.g_give_up.set_sensitive(gtk.FALSE)

        self.g_show = gu.bButton(self.action_area, _("Show"), self.show_answer)
        self.g_show.set_sensitive(gtk.FALSE)
        self.practise_box.show_all()
        ##############
        # config_box #
        ##############
        self.config_box.set_spacing(gnome.uiconsts.PAD_SMALL)
        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
            self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
        self.g_lessonfile_selector.show()
        self.config_box.pack_start(self.g_lessonfile_selector)
        # ------------------------------------------
        self._add_auto_new_question_gui(self.config_box)
        # ----------------------------------------------
        ##############
        # statistics #
        ##############
        self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                   _("Identify by name"))

        self.update_gui_after_lessonfile_change()
    def show_answer(self, widget=None):
        if self.m_t.q_status in (const.QSTATUS_SOLVED, const.QSTATUS_GIVE_UP):
            self.g_win.display_in_musicviewer(self.m_t.m_P.get_music())
    def update_answer_buttons(self):
        if self.m_t.m_P:
            self.g_bb.initialize(self.m_t.m_P.header.fillnum,
                                 self.m_t.m_P.header.filldir)
            for k in self.m_t.m_P.m_names_in_file_order:
                self.g_bb.add(k, self.m_t.m_P.m_names[k],
                      self.m_t.m_statistics.get_label_style(), self.on_click)
        else:
            self.g_bb.initialize(0, 0)
    def update_gui_after_lessonfile_change(self):
        if self.m_t.m_P:
            self.g_new.set_sensitive(gtk.TRUE)
        else:
            self.g_new.set_sensitive(gtk.FALSE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_show.set_sensitive(gtk.FALSE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.update_answer_buttons()
        self.g_win.set_title("Solfege - " + self.get_pretty_name())
    def give_up(self, _o=None):
        if self.m_t.q_status == const.QSTATUS_WRONG:
            self.g_flashbar.push(_("The answer is: %s") % self.m_t.m_P.get_name())
            self.m_t.give_up()
            self.g_show.set_sensitive(gtk.TRUE)
            self.g_new.set_sensitive(gtk.TRUE)
            self.g_give_up.set_sensitive(gtk.FALSE)
    def on_click(self, button):
        if self.m_t.q_status == const.QSTATUS_NO:
            self.g_flashbar.flash(_("Click 'New' to begin."))
            return
        if self.m_t.q_status == const.QSTATUS_SOLVED:
            if self.m_t.guess_answer(button.get_data('pcode')):
                self.g_flashbar.flash(_("Correct, but you have already solved this question"))
            else:
                self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
        elif self.m_t.q_status in (const.QSTATUS_NEW, const.QSTATUS_WRONG):
            if self.m_t.guess_answer(button.get_data('pcode')):
                self.g_flashbar.flash(_("Correct"))
                self.g_new.set_sensitive(gtk.TRUE)
                self.g_show.set_sensitive(gtk.TRUE)
                self.g_give_up.set_sensitive(gtk.FALSE)
            else:
                self.g_flashbar.flash(_("Wrong"))
                if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
                    self.m_t.play_question()
                self.g_give_up.set_sensitive(gtk.TRUE)
    def new_question(self, widget=None):
        g = self.m_t.new_question()
        if g == self.m_t.OK:
            self.g_repeat.set_sensitive(gtk.TRUE)
            self.g_new.set_sensitive(
                not self.get_bool('config/picky_on_new_question'))
            self.g_give_up.set_sensitive(gtk.FALSE)
            self.g_show.set_sensitive(gtk.FALSE)
            self.g_flashbar.clear()
            self.m_t.play_question()
    def on_start_practise(self):
        self.m_t.m_statistics.reset_session()
        gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
            self.g_flashbar.flash(_("Click 'New' to begin.")))
    def on_end_practise(self):
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_show.set_sensitive(gtk.FALSE)
        self.g_new.set_sensitive(gtk.TRUE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.g_flashbar.clear()
        self.m_t.end_practise()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# gnome emu

import gtk
from uiconsts import *
import uiconsts
from src import gu
from src.i18n import _
def _add_help(self, a, b):
    print "add_help not implemented"
gtk.GtkWidget.add_help = _add_help

class GnomeApp(gtk.GtkWindow):
    def __init__(self, a, b):
        gtk.GtkWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
        self._cid = 4
        self.__vbox = gtk.GtkVBox()
        self.add(self.__vbox)
        self.__menubox = gu.bHBox(self.__vbox, gtk.FALSE)
        self.__toolbarbox = gu.bHBox(self.__vbox, gtk.FALSE)
        self.__contentsbox = gu.bVBox(self.__vbox)
        self.__appbarbox = gu.bHBox(self.__vbox, gtk.FALSE)
        self.__dockitems = {}
    def show(self):
        self.__vbox.show()
        self.__menubox.show()
        self.__toolbarbox.show()
        self.__contentsbox.show()
        self.__appbarbox.show()
        gtk.GtkWindow.show(self)
    def flash(self, msg):
        self.__appbar.g_statusbar.push(self._cid, msg)
        gtk.timeout_add(2000, self._pop_message, self._cid)
        self._cid = self._cid + 1
    def _pop_message(self, cid):
        self.__appbar.g_statusbar.pop(cid)
    def set_statusbar(self, bar):
        self.__appbarbox.pack_start(bar)
        bar.show()
        self.__appbar = bar
    def set_contents(self, c):
        self.__contentsbox.pack_start(c)
    def create_menus(self, uiinfo):
        hdlbox = gtk.GtkHandleBox()
        hdlbox.show()
        self.__dockitems['Menubar'] = hdlbox

        self.__menubox.pack_start(hdlbox, gtk.TRUE, gtk.TRUE)
        ag = gtk.GtkAccelGroup()
        itemf = gtk.GtkItemFactory(gtk.GtkMenuBar.get_type(), "<main>", ag)
        self.add_accel_group(ag)
        v = []
        def func(data, item, self=self):
            item.set_data('uidata', self._m_cbdict[data][1])
            self._m_cbdict[data][0](item)
        idx = 0
        self._m_cbdict = {}
        for menu in uiinfo:
            v.append(("/"+menu[1], None, None, 0, "<Branch>"))
            for m in menu[3]:
                if m[0] == APP_UI_ITEM:
                    v.append(("/"+menu[1]+"/"+m[1], "<ctrl>a", func, idx, ""))
                    self._m_cbdict[idx] =  (m[3], m[4])
                    idx = idx + 1
                elif m[0] == APP_UI_ITEM_CONFIGURABLE:
                    label = {
                     uiconsts._APP_CONFIGURABLE_ITEM_PREFERENCES: _("Preferences"),
                     uiconsts._APP_CONFIGURABLE_ITEM_EXIT: _("Quit"),
                     uiconsts._APP_CONFIGURABLE_ITEM_ABOUT: _("About")}[m[7]]
                    v.append(("/"+menu[1]+"/"+label, None, func, idx, ""))
                    self._m_cbdict[idx] = (m[3], m[4])
                elif m[0] == APP_UI_SEPARATOR:
                    v.append(("/"+menu[1]+"/sep%i"%idx, None, func, 0, "<Separator>"))
                idx = idx + 1
        itemf.create_items(v)
        mb = itemf.get_widget("<main>")
        hdlbox.add(mb)
        mb.show()
    def add_toolbar(self, toolbar, name, behavior, placement, band_num,
                    band_position, offset):
        hdlbox = gtk.GtkHandleBox()
        hdlbox.show()
        self.__dockitems[name] = hdlbox
        self.__toolbarbox.pack_start(hdlbox)
        self.__toolbarbox.reorder_child(hdlbox, band_position)
        hdlbox.add(toolbar)
        toolbar.show()
    def get_dock_item_by_name(self, name):
        return self.__dockitems[name]

class GnomePropertyBox(gtk.GtkDialog):
    def __init__(self):
        gtk.GtkDialog.__init__(self)
        self.vbox.set_border_width(uiconsts.PAD)
        self.vbox.set_spacing(uiconsts.PAD)
        self.g_notebook = gtk.GtkNotebook()
        self.g_notebook.show()
        self.vbox.pack_start(self.g_notebook)
        self.g_ok = gu.bButton(self.action_area, _("Ok"), self._ok)
        self.g_ok.set_sensitive(gtk.FALSE)
        self.g_apply = gu.bButton(self.action_area, _("Apply"), self._apply)
        self.g_apply.set_sensitive(gtk.FALSE)
        self.g_cancel = gu.bButton(self.action_area, _("Close"), self.destroy)
        self.g_help = gu.bButton(self.action_area, _("Help"), self.on_help)
    def connect(self, event, cb):
        if event in ('apply', 'help'):
                pass
        else:
                gtk.GtkDialog.connect(self, event, cb)
    def append_page(self, page, label):
        self.g_notebook.append_page(page, label)
    def set_modified(self, m):
        self.g_ok.set_sensitive(gtk.TRUE)
        self.g_apply.set_sensitive(gtk.TRUE)
    def _ok(self, widget):
        self._apply(widget)
        self.destroy()
    def _apply(self, widget):
        for x in range(len(self.g_notebook.children())):
            self.on_apply(self.g_notebook.children()[x], x)
        self.on_apply(self.g_notebook, -1)
        self.g_apply.set_sensitive(gtk.FALSE)

class GnomeAppBar(gtk.GtkHBox):
    def __init__(self):
        gtk.GtkHBox.__init__(self)
        self.g_statusbar = gtk.GtkStatusbar()
        self.g_statusbar.show()
        self.pack_start(self.g_statusbar, gtk.TRUE, gtk.TRUE)
        self.g_progressbar = gtk.GtkProgressBar()
        self.g_progressbar.show()
        self.pack_start(self.g_progressbar)
    def set_progress(self, p):
        self.g_progressbar.set_percentage(p)
    def pop(self):
        self.g_statusbar.pop(1)
    def push(self, msg):
        self.g_statusbar.push(msg, 1)

class GnomeCanvas(gtk.GtkLabel):
    def __init__(self):
        gtk.GtkLabel.__init__(self, "Not on windows")
        self.show()
    def root(self): pass

def GnomeStock(name):
    return gtk.GtkPixmap(None, 'gnomeemu/xpm/%s.xpm' % name, None)

def GnomeStockButton(pixmapname):
    p = GnomeStock(pixmapname)
    p.show()
    b = gtk.GtkButton()
    b.add(p)
    return b

def GnomeAbout(name, version, copyright, authors, text):
    w = gtk.GtkDialog()
    w.vbox.set_border_width(uiconsts.PAD_SMALL)
    w.vbox.set_spacing(uiconsts.PAD_SMALL)

    frame = gtk.GtkFrame()
    w.vbox.pack_start(frame)
    mbox = gtk.GtkVBox()
    mbox.set_border_width(uiconsts.PAD_SMALL)
    mbox.set_spacing(uiconsts.PAD_SMALL)
    frame.add(mbox)
    
    label = gtk.GtkLabel(name + " " + version)
    label.set_name("AboutHeading")
    evbox = gtk.GtkEventBox()
    evbox.set_name('AboutHeadingBox')
    vbox = gtk.GtkHBox()
    evbox.add(vbox)
    vbox.set_border_width(uiconsts.PAD)
    vbox.pack_start(label)
    mbox.pack_start(evbox, gtk.FALSE)

    evbox = gtk.GtkEventBox()
    evbox.set_name("AboutBodyBox")
    bodybox = gtk.GtkVBox()
    bodybox.set_border_width(uiconsts.PAD)
    evbox.add(bodybox)
    mbox.pack_start(evbox)

    label = gtk.GtkLabel(copyright)
    label.set_name("Heading2")
    label.set_alignment(0.0, 0.0)
    bodybox.pack_start(label)

    ahbox = gtk.GtkHBox()
    bodybox.pack_start(ahbox)
    
    label = gtk.GtkLabel(_("Authors:"))
    label.set_alignment(0.0, 0.0)
    label.set_name("Heading3")
    ahbox.pack_start(label, gtk.FALSE)

    avbox = gtk.GtkVBox()
    ahbox.pack_start(avbox)

    for author in authors:
        label = gtk.GtkLabel(author)
        label.set_alignment(0.0, 0.0)
        avbox.pack_start(label)

    label = gtk.GtkLabel(text)
    label.set_alignment(0.0, 1.0)
    label.set_justify(gtk.JUSTIFY_LEFT)
    bodybox.pack_start(label)
    b = gtk.GtkButton(_("Close"))
    b.connect('clicked', w.destroy)
    w.action_area.pack_start(b)
    w.show_all()
    return w

class GnomeErrorDialog(gtk.GtkDialog):
    def __init__(self, msg, parent=None):
        gtk.GtkDialog.__init__(self, parent)
        self.vbox.set_border_width(uiconsts.PAD)
        label = gtk.GtkLabel(msg)
        self.vbox.pack_start(label)
        b = gtk.GtkButton(_("Close"))
        self.action_area.pack_start(b)
        b.connect('clicked', self.close)
        self.set_modal(gtk.TRUE)
        self.show_all()
    def close(self, *v):
        self.destroy()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import gtk
import GDK
import re, string

class GtkXmHTML(gtk.GtkScrolledWindow):
    def __init__(self):
        gtk.GtkScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.g_text = gtk.GtkText()
        self.g_text.set_word_wrap(gtk.TRUE)
        self.add(self.g_text)
        self.g_text.show()
        self.g_text.connect('event', self.on_event)
        self.g_text.set_editable(gtk.FALSE)
    def on_event(self, widget, event):
        if event.type == GDK.BUTTON_RELEASE and event.button == 3:
            href = self.get_href(self.g_text.get_position())
            if href:
              self.activate_cb(href)
    def get_href(self, idx):
        while idx > 0 and self.g_text.get_chars(idx, idx+1) not in  ["[", "]"]:
            idx = idx - 1
        if self.g_text.get_chars(idx, idx + 1) == ']':
            return
        r = re.compile("\[(.*?)\]")
        m = r.match(self.g_text.get_chars(idx, -1))
        return m.group(1)
    def source(self, txt):
        txt = self.html2text(txt)
        if self.g_text.get_length() > 0:
            self.g_text.delete_text(0, self.g_text.get_length())
        self.g_text.insert_defaults(txt)
    def html2text(self, html):
        # ugly, but simple
        if re.match("<html><body>\s*<pre>.*?GNU GENERAL PUBLIC LICENSE", html, re.MULTILINE|re.DOTALL):
            m = re.match("<html><body>\s*<pre>(.*?)</pre>\s*</body></html>", html, re.MULTILINE|re.DOTALL)
            if m:
                return m.group(1)
        whitespace_re = re.compile(r"(?s)(\s)")
        comment_re = re.compile(r"(?s)<!--.*?-->")
        html_re = re.compile(r"<.*?>")
        href_re = re.compile(r"""(?s)<a\s*href\s*=\s*"(.*?)"\s*>(.*?)</a>""")
        hx_re = re.compile(r"<h(\d)>(.*?)</h\1>")
        title_re = re.compile("<title>.*?</title>", re.DOTALL)
        html = comment_re.sub("", html)
        while 1:
            m = href_re.search(html)
            if not m:
                break
            html = html[:m.start()] + "%s[%s]" % (m.group(2), m.group(1)) + html[m.end():]
        while 1:
            m = hx_re.search(html)
            if not m:
                break
            html = html[:m.start()] \
                   + "{br}{br}%s{br}" % m.group(2) \
                   + "=" * len(m.group(2)) \
                   + "{br}" + html[m.end():]
        for old, new in (("<p>", "{p}"), ("<li>", "{li}"),
                         ("<hr>", "{hr}"), ("<br>", "{br}")):
            html = string.replace(html, old, new)
        html = title_re.sub("", html)
        html = html_re.sub("", html)
        html = whitespace_re.sub(" ", html)
        for old, new in (("{p}", "\n\n"), ("{li}", "\n\n * "),
                         ("{hr}", "-"*60), ("{br}", "\n")):
            html = string.replace(html, old, new)
        for old, new in (("&lt;", "<"), ("&gt;", ">")):
            html = string.replace(html, old, new)
        return html




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome.xmhtml
import string, re, os
import pyscript
import configureoutput

class _HtmlWidgetCommon:
    def on_key_press_event(self, *argv):
        """
        Will be used to implement scrolling when GtkHTML is available
        """
        pass
    def read_docfile(self, filename, locale=None):
        import gettext
        filefound = None
        if locale:
            languages = [locale]
        else:
            languages = gettext.lang
        for lang in languages:
            if os.path.isfile(os.path.join('online-docs', lang, filename)):
                os.chdir(os.path.join("online-docs", lang))
                self.source(open(filename, 'r').read())
                filefound = 1
                os.chdir("../..")
                break
        if not filefound:
            for lang in languages:
                fn = os.path.join('online-docs', lang, 'missing-helpfile.html')
                if os.path.isfile(fn):
                    self.source(open(fn, 'r').read())
                    break

class GtkHTML(gtk.GtkScrolledWindow, _HtmlWidgetCommon):
    def __init__(self, activate_cb, anchor_track_cb):
        gtk.GtkScrolledWindow.__init__(self)
        self.__html = gtkhtml.GtkHTML()
        self.__html.show()
        self.add(self.__html)
        self.m_pyscript = pyscript.PyScript()
        self.__html.connect('link_clicked', lambda _o, url, f=activate_cb: f(url))
        self.__html.connect('url_requested', self.on_url_requested)
        if anchor_track_cb:
            self.__html.connect('on_url', lambda _o, url, f=anchor_track_cb:
                                f(url))
        self.__html.connect('load_done', self.on_load_done)
    def on_load_done(self, html, *v):
        #print "on_load_done", html, v
        pass
    def on_url_requested(self, html, fn, handle):
        """This is called when the widget need to load images
        """
        s = open(fn, 'r').read()
        self.__html.write(handle, s)
        self.__html.end(handle, gtkhtml.HTML_STREAM_OK)
    def source(self, txt):
        txt = self.m_pyscript.evaluate(txt)
        self.m_handle = self.__html.begin()
        self.__html.write(self.m_handle, txt)
        self.__html.end(self.m_handle, gtkhtml.HTML_STREAM_OK)

class XmHTML(gtk.GtkEventBox, _HtmlWidgetCommon):
    """
    I had to make SolfegeXmHTML a GtkVBox containing a SolfegeXmHTML because
    otherwise there are problems with redrawing.
    """
    def __init__(self, activate_cb, anchor_track_cb):
        gtk.GtkEventBox.__init__(self)
        self.__html = gnome.xmhtml.GtkXmHTML()
        self.__html.show()
        self.m_pyscript = pyscript.PyScript()
        self.activate_cb = activate_cb
        self.anchor_track_cb = anchor_track_cb
        # for the win32 port
        if configureoutput.WITH_GNOME:
            self.__html.connect('activate',
                lambda _o, cbstruct, f=activate_cb: f(cbstruct.href))
            if anchor_track_cb:
                self.__html.connect('anchor_track',
                  lambda _o, cbstruct, F=anchor_track_cb: F(cbstruct.href))
        else:
            self.__html.activate_cb = self.activate_cb
        self.add(self.__html)
    def source(self, txt):
        txt = self.m_pyscript.evaluate(txt)
        self.__html.source(txt)

if configureoutput.WITH_GTKHTML:
    import gtkhtml
    HtmlWidget = GtkHTML
else:
    HtmlWidget = XmHTML



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from __future__ import nested_scopes

import sys
import time, random
from i18n import _
import configureoutput

def do_some_cmdline_options(argv):
        if '-h' in argv or '--help' in argv:
            print """
GNU Solfege %s
Type "man solfege" for more details.
Options:
  --no-sound             Do not play any sounds. Useful when porting.
  --verbose-sound-init   display more info about the sound setup
  -v, --version
  -w, --warranty         show warranty and copyright
  -h, --help             Display this message
  --gtk-help             Display the usual gtk/gnome commandline options.
    """ % configureoutput.VERSION_STRING
            sys.exit()

        if '-v' in argv or '--version' in argv:
            print """GNU Solfege %s
This is free software. It is covered by the GNU General Public License,
and you are welcome to change it and/or distribute copies of it under
certain conditions. Invoke as `solfege --warranty` for more information.

Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen <tca@gnu.org>
            """ % configureoutput.VERSION_STRING
            sys.exit()

        if '-w' in argv or '--warranty' in argv:
            print """GNU Solfege %s
Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen <tca@gnu.org>

    This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

    You should have received a copy (refer to the file COPYING) of the
GNU General Public License along with this program; if not, write to
the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
USA.
        """ % configureoutput.VERSION_STRING
            sys.exit()


do_some_cmdline_options(sys.argv)

if '--gtk-help' in sys.argv:
    del sys.argv[sys.argv.index('--gtk-help')]
    sys.argv.append('--help')

# silent, GNOME, be silent!
sys.argv.append('--disable-sound')

import locale
import gtk

gtk.rc_parse("solfege.gtkrc")

class SplashWin(gtk.GtkWindow):
    def __init__(self):
        gtk.GtkWindow.__init__(self, gtk.WINDOW_POPUP)
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_policy(gtk.FALSE, gtk.FALSE, gtk.FALSE)
        self.set_name("SplashWindow")
        frame = gtk.GtkFrame()
        frame.set_shadow_type(gtk.SHADOW_OUT)
        self.add(frame)
        vbox = gtk.GtkVBox()
        vbox.set_border_width(20)
        frame.add(vbox)
        l = gtk.GtkLabel(_("Starting GNU Solfege %s") % configureoutput.VERSION_STRING)
        l.set_name("Heading1")
        vbox.pack_start(l)
        l = gtk.GtkLabel("http://www.solfege.org")
        vbox.pack_start(l)
        self.show_all()

if not filter(lambda a: a in sys.argv, ['--no-splash', '--usage', '-?']):
    splash_win = SplashWin()
    time.sleep(0.1)
    gtk.gdk_flush()
    while gtk.events_pending():
        gtk.mainiteration(0)
else:
    splash_win = None
    if '--no-splash' in sys.argv:
        del sys.argv[sys.argv.index('--no-splash')]

# we have to remove all options gnome/gtk don't understand
# from sys.argv
args = []
for a in ('--no-sound', '--verbose-sound-init'):
    if a in sys.argv:
        args.append(a)
        del sys.argv[sys.argv.index(a)]

import gnome, gnome.ui
# FIXME
from gnome.ui import *
import GDK, os, const, string
locale.setlocale(locale.LC_NUMERIC, "C")
import app
from htmlwidget import HtmlWidget
import utils, history
from configwindow import ConfigWindow
import mpd, mpd.musicdisplayer, gu, cfg
import tracebackwindow


# redirect error messages to a window that will popup if
# something bad happens.

w = tracebackwindow.TracebackWindow()
sys.stderr = w


class DocViewer(gtk.GtkVBox):
    def __init__(self, activate_cb):
        gtk.GtkVBox.__init__(self)
        self.m_htmlwidget = HtmlWidget(activate_cb, self.on_anchor_track)
        self.m_htmlwidget.show_all()
        self.pack_start(self.m_htmlwidget)
        self.set_usize(300, 300)
        self.read_docfile = self.m_htmlwidget.read_docfile
        self.on_key_press_event = self.m_htmlwidget.on_key_press_event
        self.m_statusbar = gtk.GtkStatusbar()
        self.m_statusbar.show()
        self.pack_start(self.m_statusbar, gtk.FALSE)
        self.m_link_under_mouse = None
        self.source = self.m_htmlwidget.source
    def on_anchor_track(self, url):
        if url:
            # remove newlines in url because it make the window resize
            s = string.replace(url, "\n", "")
            # we have to use m_link_under_mouse because XmHTML does not
            # always emit the correct signals. Try to go without this
            # when we switch to GtkHTML
            if self.m_link_under_mouse:
                self.m_statusbar.pop(1)
            self.m_statusbar.push(1, s)
            self.m_link_under_mouse = 1
        else:
            self.m_statusbar.pop(1)
            self.m_link_under_mouse = None


class MusicViewerWindow(gtk.GtkDialog):
    def __init__(self, app):
        gtk.GtkDialog.__init__(self)
        self.m_app = app
        self.set_usize(400, 200)
        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.vbox.pack_start(self.g_music_displayer)
        gu.bButton(self.action_area, _("Close"), self.m_app.close_musicviewer)
        self.connect('destroy', self.m_app.close_musicviewer)
        self.show_all()
    def display_music(self, music):
        fontsize = cfg.get_int('config/feta_font_size=20')
        self.g_music_displayer.display(music, fontsize)

class MainWin(gnome.ui.GnomeApp, cfg.ConfigUtils):
    def __init__(self, argv):
        gnome.ui.GnomeApp.__init__(self,
          'solfege%i.%i' % (configureoutput.MAJOR_VERSION,
                            configureoutput.MINOR_VERSION), 'Solfege')
        cfg.ConfigUtils.__dict__['__init__'](self, 'mainwin')
        if self.get_bool('gui/mainwin_user_resizeable'):
            self.set_policy(0, 1, 0)
        else:
            self.set_policy(0, 0, 1)
        self.connect('destroy', self.quit_program)
        self.connect('key_press_event', self.on_key_press_event)
        self.g_about_window = None
        self.main_box = gtk.GtkVBox()
        self.main_box.show()
        self.set_contents(self.main_box)
        self.m_exercise = None
        self.m_viewer = None
        self.box_dict = {}
        self.g_config_window = None
        self.g_musicviewer_window = None
        self.m_history = history.History()
        self.m_key_bindings = {'history_back_ak': self.history_back,
                               'history_forward_ak': self.history_forward,
                               'history_reload_ak': self.history_reload}
        self.m_app = app.SolfegeApp(argv, self)
        self.setup_menubar()
        self.setup_toolbars()
        if configureoutput.WITH_GNOME:
            self.m_app.handle_href('welcome.html')
        else:
            self.m_app.handle_href('nognomewelcome.html')
    def display_error_message(self, msg):
        gnome.ui.GnomeErrorDialog(msg)
    def show_about_window(self, widget):
        if not self.g_about_window:
            self.g_about_window = gnome.ui.GnomeAbout(
                    'GNU Solfege', configureoutput.VERSION_STRING,
                    'Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen',
                    ['Tom Cato Amundsen',
                     'Oliver Vollmer %s' % _("(toolbar icons)"),
                     'Giovanni Chierico %s' % _("(some lessonfiles)"),
                     'Joe Lee %s' % _("(sound code for the MS Windows port)"),
                     ],
                    #['see AUTHORS file or online documentation for details'],
                    _("GPL'ed eartraining.") + "\n"
                    + _("New version available at http://www.solfege.org"))
            self.g_about_window.connect('destroy', self.hide_about_window)
            self.g_about_window.show()
    def hide_about_window(self, widget):
        self.g_about_window.destroy()
        self.g_about_window = None
    def post_constructor(self):
        if not self.get_bool("gui/navbar_visible"):
            self.get_dock_item_by_name('backandforward').hide()
        if not self.get_bool("gui/toolbar_visible"):
            self.get_dock_item_by_name('main-toolbar').hide()
    def activate_exercise(self, href):
        protocol, action, exname, collection, lessonfile, config = \
           utils.parse_url(href)
        assert protocol == 'solfege'
        if self.m_viewer:
            if self.m_viewer != 'docviewer':
                self.box_dict[self.m_viewer].on_end_practise()
            self.box_dict[self.m_viewer].hide()
        self.box_dict[exname].show()
        self.box_dict[exname].on_start_practise()
        self.m_viewer = exname
        if action in ['practise', 'config', 'statistics']:
            # We need this test because not all exercises use a notebook.
            if self.box_dict[self.m_viewer].g_notebook:
                self.box_dict[self.m_viewer].g_notebook.set_page(
                   ['practise', 'config', 'statistics'].index(action))
        self.set_title("Solfege - " + self.box_dict[exname].get_pretty_name())
        self.m_history.add(href)
    def _show_docviewer(self):
        if not self.box_dict.has_key('docviewer'):
            self.box_dict['docviewer'] = DocViewer(self.m_app.handle_href)
            self.main_box.pack_start(self.box_dict['docviewer'])
        if self.m_viewer and (self.m_viewer != 'docviewer'):
                self.box_dict[self.m_viewer].hide()
        self.m_viewer = 'docviewer'
        self.box_dict['docviewer'].show()
    def display_docfile(self, fn):
        self._show_docviewer()
        self.box_dict['docviewer'].read_docfile(fn)
        self.set_title("Solfege - %s" % fn)
        self.m_history.add(fn)
    def display_html(self, html):
        self._show_docviewer()
        self.box_dict['docviewer'].source(html)
    def history_back(self, widget=None):
        self.m_history.back()
        self.m_history.lock()
        self.m_app.handle_href(self.m_history.get_current())
        self.m_history.unlock()
    def history_forward(self, widget=None):
        self.m_history.forward()
        self.m_history.lock()
        self.m_app.handle_href(self.m_history.get_current())
        self.m_history.unlock()
    def history_reload(self, widget=None):
        self.m_history.lock()
        self.m_app.handle_href(self.m_history.get_current())
        self.m_history.unlock()
    def initialise_exercise(self, teacher):
        assert not self.box_dict.has_key(teacher.m_exname)
        n = utils.exercise_name_to_module_name(teacher.m_exname)
        exec("import %s" % n)
        self.box_dict[teacher.m_exname] = locals()[n].Gui(teacher, self)
        self.main_box.pack_start(self.box_dict[teacher.m_exname])
    def on_key_press_event(self, widget, event):
        if event.type == GDK.KEY_PRESS:
            for s in self.m_key_bindings.keys():
                if (event.state, event.keyval) \
                         == utils.parse_key_string(self.get_string(s)):
                    self.m_key_bindings[s]()
                    return 1
        self.box_dict[self.m_viewer].on_key_press_event(widget, event)
    def open_preferences_window(self, widget=None):
        if not self.g_config_window:
            self.g_config_window = ConfigWindow(self.m_app)
            self.g_config_window.set_parent(self)
            self.g_config_window.show()
        else:
            self.g_config_window.show()
    def setup_menubar(self):
        file_menu = []
        for e in const.exercises:
            file_menu.append(
             (APP_UI_ITEM, const.exercisedata[e][0], None,
              lambda w, e=e: self.m_app.handle_href("solfege:practise/"+e),
              None, APP_PIXMAP_NONE, None,
              const.exercisedata[e][2], const.exercisedata[e][3]))
        file_menu = file_menu + [
            UIINFO_SEPARATOR,
            UIINFO_MENU_PREFERENCES_ITEM(self.open_preferences_window),
            UIINFO_MENU_EXIT_ITEM(self.quit_program),
            ]
        help_menu = [
                    (APP_UI_ITEM, _("_Help on current exercise"), None,
                     lambda w: self.m_app.please_help_me(),
                     None, APP_PIXMAP_NONE, None, GDK.F1, 0),
                    (APP_UI_ITEM, _("Welcome"), None,
                    lambda w: self.m_app.handle_href('welcome.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    (APP_UI_ITEM, _("All help files"), None,
                    lambda w: self.m_app.handle_href('index.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    (APP_UI_ITEM, _("All installed lesson files"), None,
                    lambda w: self.m_app.handle_href('solfege:all-lessonfiles'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    UIINFO_SEPARATOR,
                    (APP_UI_ITEM, _("Copyright notice"), None,
                    lambda w: self.m_app.handle_href('copyright.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    (APP_UI_ITEM, _("Credits"), None,
                    lambda w: self.m_app.handle_href('AUTHORS.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    (APP_UI_ITEM, _("Mailinglists, web page etc."), None,
                    lambda w: self.m_app.handle_href('online-resources.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    (APP_UI_ITEM, _("Reporting bugs"), None,
                    lambda w: self.m_app.handle_href('bug-reporting.html'),
                    None, APP_PIXMAP_NONE, None, 0, 0),
                    UIINFO_MENU_ABOUT_ITEM(self.show_about_window, None),
                    ]
        development_menu =  [
            (APP_UI_ITEM, _("This menu contains\nmostly useless stuff"),
             None, None, None, APP_PIXMAP_NONE, None, 0, 0),
             UIINFO_SEPARATOR,
            (APP_UI_ITEM, _("_Tempo"), None,
             lambda w: self.m_app.handle_href('solfege:/identify-bpm'),
             None, APP_PIXMAP_NONE, None, 0, 0),
            (APP_UI_ITEM, _("Intervall"), None,
             lambda w: self.m_app.handle_href('solfege:/intervalls'),
             None, APP_PIXMAP_NONE, None, 0, 0),
            (APP_UI_ITEM, _("Rhythm"), None,
             lambda w: self.m_app.handle_href('solfege:/polyrhythm'),
             None, APP_PIXMAP_NONE, None, 0, 0),
            (APP_UI_ITEM, const.exercisedata['harmonic-progression-dictation'][0], None,
             lambda w: self.m_app.handle_href('solfege:/harmonic-progression-dictation'),
             None, APP_PIXMAP_NONE, None, 0, 0),

            UIINFO_ITEM_STOCK(_("Experiment"), None,
                              self.experiment, STOCK_MENU_EXEC),
                 ]
        if configureoutput.ENABLE_TUNER:
            development_menu = development_menu + [
            (APP_UI_ITEM, _("Tuner"), None,
             lambda w: self.m_app.handle_href('solfege:/tuner'),
             None, APP_PIXMAP_NONE, None, 0, 0)]
        self.m_menuinfo = menu = [
            UIINFO_SUBTREE(_("_File"), file_menu),
            UIINFO_SUBTREE(_("_Development"), development_menu),
            UIINFO_SUBTREE(_("_Help"), help_menu),
            ]
        self.create_menus(menu)
        if not self.get_bool('gui/development_menu_visible'):
            self.get_dock_item_by_name("Menubar").children()[0].children()[1].hide()
    def experiment(self, w=None):
        pass
    def quit_program(self, w=None):
        self.m_app.quit_program()
        gtk.mainquit()
    def setup_toolbars(self):
        self.g_toolbar = gtk.GtkToolbar(gtk.ORIENTATION_HORIZONTAL,
                                        gtk.TOOLBAR_ICONS)
        self.g_toolbar.set_style({'icons': gtk.TOOLBAR_ICONS,
                                  'text': gtk.TOOLBAR_TEXT,
                                  'both': gtk.TOOLBAR_BOTH}
               [self.get_string('gui/toolbar_style')])
        bg = self.get_style().bg[gtk.STATE_NORMAL]
        for i in const.exercises:
            n = string.replace(const.exercisedata[i][0], '_', '')
            pix, mask = gtk.create_pixmap_from_xpm(self, None,
               os.path.join('xpm', const.exercisedata[i][1]))
            self.g_toolbar.append_item(n, n, i, gtk.GtkPixmap(pix, mask),
                    lambda w, i=i: self.m_app.handle_href("solfege:practise/"+i))
        self.add_toolbar(self.g_toolbar, "main-toolbar",
                         gnome.uiconsts.DOCK_ITEM_BEH_NORMAL,
                         gnome.uiconsts.DOCK_TOP, 1, 1, 0)
        self.g_navbar = gtk.GtkToolbar(gtk.ORIENTATION_HORIZONTAL,
                                       gtk.TOOLBAR_ICONS)
        self.g_navbar.append_item(_("Back"), None, None,
            gnome.ui.GnomeStock(gnome.ui.STOCK_PIXMAP_BACK), self.history_back)
        self.g_navbar.append_item(_("Forward"), None, None,
              gnome.ui.GnomeStock(gnome.ui.STOCK_PIXMAP_FORWARD),
                                  self.history_forward)
        self.g_navbar.set_style({'icons': gtk.TOOLBAR_ICONS,
                                 'text': gtk.TOOLBAR_TEXT,
                                 'both': gtk.TOOLBAR_BOTH}
               [self.get_string('gui/navbar_style')])
        self.add_toolbar(self.g_navbar, 'backandforward',
                         gnome.uiconsts.DOCK_ITEM_BEH_NORMAL,
                         gnome.uiconsts.DOCK_TOP, 1, 0, 0)
    def display_in_musicviewer(self, music):
        if not self.g_musicviewer_window:
            self.g_musicviewer_window = MusicViewerWindow(self)
            self.g_musicviewer_window.show()
        self.g_musicviewer_window.display_music(music)
    def close_musicviewer(self, widget=None):
        self.g_musicviewer_window.destroy()
        self.g_musicviewer_window = None

w = MainWin(args)
w.show()
w.post_constructor()
if splash_win:
    splash_win.destroy()
gtk.mainloop()




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os

import gtk, gnome
import gu, widgets
import mpd, mpd.musicdisplayer
import abstract, const, lessonfile
from i18n import _
from soundcard.rat import Rat
import soundcard
import utils
import configureoutput

class Teacher(abstract.LessonbasedTeacher):
    def __init__(self, exname, app):
        abstract.LessonbasedTeacher.__init__(self, exname, app, const.USE_DICTATION)
        self.lessonfileclass = lessonfile.DictationLessonfile
        self.parse_lessonfile()
        #to please abstract.LessonParser
        self.q_status = const.QSTATUS_NEW
    def parse_lessonfile(self):
        abstract.LessonbasedTeacher.parse_lessonfile(self)

class Gui(abstract.LessonbasedGui):
    def __init__(self, teacher, window):
        abstract.LessonbasedGui.__init__(self, teacher, window)
        self.m_key_bindings = {'play_ak': self.play,
                               'show_ak': self.show_answer}
        ################
        # practise_box #
        ################
        self.g_question_title = gu.bLabel(self.practise_box, "")

        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.g_music_displayer.set_usize(-1, 150)
        self.g_music_displayer.show()
        self.practise_box.pack_start(self.g_music_displayer)
        ###############
        # action_area #
        ###############
        self.g_partbox = gu.bHBox(self.practise_box, gtk.FALSE)
        b = gnome.ui.GnomeStockButton("Back")
        b.connect('clicked', self.select_previous)
        b.show()
        if configureoutput.WITH_GNOME:
            b.children()[0].children()[0].children()[1].hide()
        self.action_area.pack_start(b, gtk.FALSE)
        b = gnome.ui.GnomeStockButton("Forward")
        b.show()
        b.connect('clicked', self.select_next)
        if configureoutput.WITH_GNOME:
            b.children()[0].children()[0].children()[1].hide()
        self.action_area.pack_start(b, gtk.FALSE)
        self.g_play = gu.bButton(self.action_area, _("Play the whole music"),
                                 self.play)
        self.g_show = gu.bButton(self.action_area, _("Show"), self.show_answer)
        ##############
        # config_box #
        ##############
        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
              self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
        self.g_lessonfile_selector.show()
        self.config_box.pack_start(self.g_lessonfile_selector)
        self.update_gui_after_lessonfile_change()
    def select_previous(self, widget):
        self.m_t.m_P.select_previous()
        self.display_start_of_music()
        self._update()
    def select_next(self, widget):
        self.m_t.m_P.select_next()
        self.display_start_of_music()
        self._update()
    def play(self, widget=None):
        self.m_t.play_question()
    def on_end_practise(self):
        self.update_gui_after_lessonfile_change()
    def show_answer(self, widget=None):
        if self.m_t.m_P:
            self.g_music_displayer.display(self.m_t.m_P.get_music(),
                            self.get_int('config/feta_font_size=20'))
    def _update(self):
        """
        Updates the buttons above the action_area where you have
        one or more buttons with a small note pixmap on. Each of the
        buttons will play one part of the music in the question.
        """
        self.g_question_title.set_text(self.m_t.m_P.get_name())
        # tmp func used as callback function
        def f(w, start, end, self=self):
            track = mpd.music_to_track(self.m_t.m_P.get_music(),
                       self.get_int('config/preferred_instrument_velocity'),
                       start, end)
            apply(track.prepend_bpm, self.m_t.m_P.get_tempo())
            track.prepend_patch(0, self.get_int('config/preferred_instrument'))
            soundcard.synth.play_track(track)
        self.g_partbox.foreach(lambda o: o.destroy())
        # if the user selects a collection without lessonfiles, or if
        # the lessonfile was invalid, m_P could be None
        if self.m_t.m_P:
            v = self.m_t.m_P.get_breakpoints()
            if v == []:
                # we display one button that will play the whole music if
                # there are not breakpoints in the music
                btn = self.create_pixmap_button()
                btn.connect('clicked', f, None, None)
                btn.show()
                self.g_partbox.pack_start(btn)
                return
            tmp = [Rat(0, 1)] + v + [Rat(2**30, 1)]
            for i in range(len(tmp) - 1):
                btn = self.create_pixmap_button()
                btn.show()
                btn.connect('clicked', f, tmp[i], tmp[i+1])
                self.g_partbox.pack_start(btn)
    def display_start_of_music(self):
        fontsize = self.get_int('config/feta_font_size=20')
        try:
            if self.m_t.m_P.get_clue_music():
                self.g_music_displayer.display(self.m_t.m_P.get_clue_music(),
                                               fontsize)
            elif self.m_t.m_P.get_clue_end():
                self.g_music_displayer.display_range(self.m_t.m_P.get_music(),
                               fontsize, Rat(0, 1), self.m_t.m_P.get_clue_end())
            else:
                self.g_music_displayer.display(self.m_t.m_P.get_music(),
                                               fontsize, mpd.FIRST)
        except mpd.musicalpitch.InvalidNotenameException, n:
            gnome.ui.GnomeErrorDialog(
                    "In lessonfile '%s':\n Invalid notename: %s" % (
                    self.get_string('lessonfile'), n.m_notename) )
    def update_gui_after_lessonfile_change(self):
        self.m_t.q_status = const.QSTATUS_NEW
        if self.m_t.m_P:
            self.action_area.set_sensitive(gtk.TRUE)
            self.display_start_of_music()
        else:
            self.action_area.set_sensitive(gtk.FALSE)
            self.g_music_displayer.clear()
        self._update()
        self.g_win.set_title("Solfege - " + self.get_pretty_name())
    def create_pixmap_button(self):
        pix, mask = gtk.create_pixmap_from_xpm(self, None,
                os.path.join("xpm", "rhythm-c4.xpm"))
        pixmap = gtk.GtkPixmap(pix, mask)
        btn = gtk.GtkButton()
        btn.add(pixmap)
        pixmap.show()
        return btn




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import GDK
import mpd, gu, cfg
import types
from i18n import _

class SimplestNotenameSpinButton(gtk.GtkHBox):
    """
    The simplest notename spin button. The widget don't have any
    limits to what values are allowed. It won't change its own value,
    but call the callback function with the requested new value as value.

    This class is not usable on it's own, it has to be subclassed or
    used with NotenameSpinButtonController.
    """
    def __init__(self, value):
        assert type(value) == types.StringType
        gtk.GtkHBox.__init__(self)
        self.g_entry = gtk.GtkEntry()
        self.g_entry.set_text(value)
        self.pack_start(self.g_entry, gtk.FALSE, gtk.FALSE)
        self.g_entry.set_usize(40, -1)
        self.g_entry.set_editable(gtk.FALSE)

        box = gtk.GtkVBox()
        self.pack_start(box, gtk.FALSE, gtk.FALSE)
        eb = gtk.GtkEventBox()
        eb.connect('button_press_event', self.on_arrowclick, 1)
        self.g_up = gtk.GtkArrow(gtk.ARROW_UP, gtk.SHADOW_OUT)
        eb.add(self.g_up)
        box.pack_start(eb)
        eb = gtk.GtkEventBox()
        eb.connect('button_press_event', self.on_arrowclick, -1)
        self.g_down = gtk.GtkArrow(gtk.ARROW_DOWN, gtk.SHADOW_OUT)
        eb.add(self.g_down)
        box.pack_start(eb)
        self.g_entry.connect('event', self.on_entry_event)
    def connect_request_change(self, callback):
        self.m_callback = callback
    def on_entry_event(self, entry, event):
        if event.type == GDK.KEY_PRESS:
            if event.keyval == GDK.Page_Up:
                self.change_value(12)
                return 1
            elif event.keyval == GDK.Page_Down:
                self.change_value(-12)
                return 1
            elif event.keyval == GDK.Up:
                self.change_value(1)
                return 1
            elif event.keyval == GDK.Down:
                self.change_value(-1)
                return 1
    def on_arrowclick(self, _o, event, v):
        if event.type == GDK.BUTTON_PRESS:
            assert v in (-1, 1)
            if event.button == 1:
                self.change_value(v)
            elif event.button == 3: # right button
                self.change_value(v*12)
            elif event.button == 2: # middle button
                if v == 1:
                    self.change_value(1000)
                else:
                    self.change_value(-1000)
    def get_value(self):
        return self.g_entry.get_text()
    def change_value(self, v):
        self.m_callback(self, mpd.int_to_notename(mpd.notename_to_int(self.get_value()) + v))
    def set_value(self, n):
        assert type(n) == types.StringType
        # only change the entry if the value has actually changed, to
        # avoid unnecessary 'changed' signals emitted.
        if self.g_entry.get_text() != n:
            self.g_entry.set_text(n)

class NotenameRangeController:
    def __init__(self, spin_low, spin_high, lowest_value, highest_value):
        self.g_spin_low = spin_low
        self.g_spin_low.connect_request_change(self.on_low_changed)
        self.g_spin_high = spin_high
        self.g_spin_high.connect_request_change(self.on_high_changed)
        self.m_lowest_value = lowest_value
        self.m_highest_value = highest_value
    def on_low_changed(self, widget, value):
        """
        value: the new notename STRING
        """
        if mpd.compare_notenames(value, self.g_spin_high.get_value()) > 0:
            self.g_spin_low.set_value(self.g_spin_high.get_value())
        elif mpd.compare_notenames(value, self.m_lowest_value) < 0:
            self.g_spin_low.set_value(self.m_lowest_value)
        else:
            self.g_spin_low.set_value(value)
    def on_high_changed(self, widget, value):
        if mpd.compare_notenames(value, self.g_spin_low.get_value()) < 0:
            self.g_spin_high.set_value(self.g_spin_low.get_value())
        elif mpd.compare_notenames(value, self.m_highest_value) > 0:
            self.g_spin_high.set_value(self.m_highest_value)
        else:
            self.g_spin_high.set_value(value)

class nNotenameRangeController(NotenameRangeController, cfg.ConfigUtils):
    def __init__(self, spin_low, spin_high, lowest_value, highest_value,
                 exname, name_low, name_high):
        NotenameRangeController.__init__(self, spin_low, spin_high,
                lowest_value, highest_value)
        cfg.ConfigUtils.__init__(self, exname)
        self.m_name_low = name_low
        self.m_name_high = name_high
        #self.g_spin_low.set_value(self.get_string(self.m_name_low))
        #self.g_spin_high.set_value(self.get_string(self.m_name_high))
    def set_range(self, lowest_value, highest_value):
        """
        make a separate function for NotenameRangeController if we need it.
        """
        assert mpd.compare_notenames(lowest_value, highest_value) <= 0
        self.m_lowest_value = lowest_value
        self.m_highest_value = highest_value
        if mpd.compare_notenames(lowest_value, self.g_spin_low.get_value()) > 0:
            self.set_string(self.m_name_low, lowest_value)
            self.g_spin_low.set_value(lowest_value)
        if mpd.compare_notenames(highest_value,
                                 self.g_spin_high.get_value()) < 0:
            self.set_string(self.m_name_high, highest_value)
            self.g_spin_high.set_value(highest_value)



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import gu, const, soundcard, mpd, mpd.musicdisplayer, abstract
import lessonfile, string
from i18n import _
import widgets
import utils

class Teacher(abstract.LessonbasedTeacher):
    def __init__(self, exname, app):
        abstract.LessonbasedTeacher.__init__(self, exname, app, const.USE_SING_CHORD)
        self.lessonfileclass = lessonfile.SingChordLessonfile
        self.parse_lessonfile()
    def new_question(self):
        self.q_status = const.QSTATUS_NEW
        self.m_P.select_random_question()
    def play_answer(self):
        self.play_question()
    def play_440hz(self):
        soundcard.play_note(self.get_int('config/preferred_instrument'),
                       4, 0, mpd.notename_to_int("a'"),
                       self.get_int('config/preferred_instrument_velocity'))
    def play_answer_arpeggio(self):
        if self.q_status == const.QSTATUS_NO:
            return
        m = soundcard.Track()
        m.set_bpm(self.get_int('config/default_bpm'))
        m.set_patch(0, self.get_int('config/preferred_instrument'))
        vel = self.get_int('config/preferred_instrument_velocity')
        for x in [0, 1]:
            v = self.m_P.get_satb_vector()[x]
            s = string.split(string.strip(v), " ")
            for n in s:
                if self.get_string('user/sex') == 'female':
                    m.note(4, 0, mpd.notename_to_int(n), vel)
                else:
                    m.note(4, 0, mpd.notename_to_int(n)-12, vel)
        for x in [2, 3]:
            v = self.m_P.get_satb_vector()[x]
            s = string.split(string.strip(v), " ")
            for n in s:
                if self.get_string('user/sex') == 'male':
                    m.note(4, 0, mpd.notename_to_int(n), vel)
                else:
                    m.note(4, 0, mpd.notename_to_int(n)+12, vel)
        soundcard.synth.play_track(m)


class Gui(abstract.LessonbasedGui):
    def __init__(self, teacher, window):
        abstract.LessonbasedGui.__init__(self, teacher, window)
        self.m_key_bindings = {'new_ak': self.new_question,
                               '440hz_ak': self.m_t.play_440hz,
                               'play_answer_ak': self.m_t.play_answer_arpeggio}
        ################
        # practise_box #
        ################
        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.g_music_displayer.set_usize(-1, 170)
        self.practise_box.pack_start(self.g_music_displayer)

        self.g_new = gu.bButton(self.action_area, _("New"), self.new_question)
        gu.bButton(self.action_area, _("440hz"), lambda f, s=self: s.m_t.play_440hz())
        self.g_play_answer = gu.bButton(self.action_area, _("Play answer"), lambda f, s=self: s.m_t.play_answer_arpeggio())
        self.practise_box.show_all()
        ##############
        # config_box #
        ##############
        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
             self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
        self.g_lessonfile_selector.show()
        self.config_box.pack_start(self.g_lessonfile_selector)
        self.update_gui_after_lessonfile_change()
    def new_question(self, widget=None):
        self.m_t.new_question()
        fontsize = self.get_int('config/feta_font_size=20')
        self.g_music_displayer.display(self.m_t.m_P.get_music(), fontsize)
        self.g_play_answer.set_sensitive(gtk.TRUE)
        self.m_t.play_440hz()
    def on_end_practise(self):
        self.g_play_answer.set_sensitive(gtk.FALSE)
        self.g_music_displayer.clear(2)
        self.m_t.end_practise()
    def update_gui_after_lessonfile_change(self):
        self.g_music_displayer.clear(2)
        if self.m_t.m_P:
            self.g_new.set_sensitive(gtk.TRUE)
        else:
            self.g_new.set_sensitive(gtk.FALSE)
        self.g_play_answer.set_sensitive(gtk.FALSE)
        self.g_win.set_title("Solfege - " + self.get_pretty_name())




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
You have to call initialise to get translated messages.
"""

import gettext

def _(s):
    return s

def _i(s):
    return s

def initialise(localedir):
    global _, _i
    gettext.bindtextdomain("solfege", localedir)
    gettext.textdomain("solfege")
    _ = gettext.gettext

    cat = gettext.Catalog("solfege-intervallnames", localedir)
    _i = cat.gettext



#!/usr/bin/python

# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# This script is used to launch Solfege when running
# it from the source dir without installing.

#from __future__ import nested_scopes

#import gc
#gc.set_debug(gc.DEBUG_LEAK)
#gc.disable()

import sys

if sys.version_info < (2, 1):
    print "Solfege need Python 2.1 or newer. Your python seems to be too old."
    print "Do not report bugs in Solfege before you upgrade your python interpreter."

import src.configureoutput

import src.rcfile
src.rcfile.copy_rc14_file()
import src.cfg

src.cfg.initialise(None, "default.config", "~/.solfegerc%i.%i" %
  (src.configureoutput.MAJOR_VERSION, src.configureoutput.MINOR_VERSION))

if not src.configureoutput.WITH_GNOME:
    sys.path.insert(0, "gnomeemu")

import src.i18n
src.i18n.initialise("share/locale")

import __main__, time
__main__._st = time.clock()
if '--txtui' in sys.argv:
    import src.txtui
else:
    import src.mainwin




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
from i18n import _
import sys

class TracebackWindow(gtk.GtkDialog):
    def __init__(self):
        gtk.GtkDialog.__init__(self)
        self.set_default_size(630, 400)
        self.vbox.set_border_width(8)
        label = gtk.GtkLabel(_("Program error"))
        label.set_name('Heading1')
        self.vbox.pack_start(label, gtk.FALSE)
        label = gtk.GtkLabel(_("Hm... *something* is wrong. You should report this to bug-solfege@gnu.org or solfege-devel@lists.sourceforge.net if you think this is a bug in Solfege."))
        label.set_line_wrap(gtk.TRUE)
        self.vbox.pack_start(label, gtk.FALSE)
        scrollwin = gtk.GtkScrolledWindow()
        scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.vbox.pack_start(scrollwin)
        self.g_text = gtk.GtkText()
        scrollwin.add(self.g_text)
        b = gtk.GtkButton(_("Close"))
        self.action_area.pack_start(b)
        b.connect('clicked', self.hide)
    def write(self, txt):
        sys.stdout.write(txt)
        self.show_all()
        self.g_text.insert_defaults(txt)
    def flush(self, *v):
        pass
    def close(self, *v):
        pass



"""This module allows python programs to use GNU gettext message catalogs.

Author: James Henstridge <james@daa.com.au>
(This is loosely based on gettext.pl in the GNU gettext distribution)

The best way to use it is like so:
    import gettext
    gettext.bindtextdomain(PACKAGE, LOCALEDIR)
    gettext.textdomain(PACKAGE)
    _ = gettext.gettext
    print _('Hello World')

where PACKAGE is the domain for this package, and LOCALEDIR is usually
'$prefix/share/locale' where $prefix is the install prefix.

If you have more than one catalog to use, you can directly create catalog
objects.  These objects are created as so:
    import gettext
    cat = gettext.Catalog(PACKAGE, localedir=LOCALEDIR)
    _ = cat.gettext
    print _('Hello World')

The catalog object can also be accessed as a dictionary (ie cat['hello']).

There are also some experimental features.  You can add to the catalog, just
as you would with a normal dictionary.  When you are finished, you can call
its save method, which will create a new .mo file containing all the
translations:
    import gettext
    cat = Catalog()
    cat['Hello'] = 'konichiwa'
    cat.save('./tmp.mo')

Once you have written an internationalized program, you can create a .po file
for it with "xgettext --keyword=_ fillename ...".  Then do the translation and
compile it into a .mo file, ready for use with this module.  Note that you
will have to use C style strings (ie. use double quotes) for proper string
extraction.
"""
import os, string

prefix = '/usr/local'
localedir = prefix + '/share/locale'

def _expandLang(str):
	langs = [str]
	# remove charset ...
	if '.' in str:
		langs.append(string.split(str, '.')[0])
	# also add 2 character language code ...
	if len(str) > 2:
		langs.append(str[:2])
	return langs

lang = []
for env in 'LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG':
	if os.environ.has_key(env):
		lang = string.split(os.environ[env], ':')
		lang = map(_expandLang, lang)
		lang = reduce(lambda a, b: a + b, lang)
		break
if 'C' not in lang:
	lang.append('C')

# remove duplicates
i = 0
while i < len(lang):
	j = i + 1
	while j < len(lang):
		if lang[i] == lang[j]:
			del lang[j]
		else:
			j = j + 1
	i = i + 1
del i, j

if os.environ.has_key('PY_XGETTEXT'):
	xgettext = os.environ['PY_XGETTEXT']
else:
	xgettext = None

del os, string

error = 'gettext.error'

def _lsbStrToInt(str):
	return ord(str[0]) + \
	       (ord(str[1]) << 8) + \
	       (ord(str[2]) << 16) + \
	       (ord(str[3]) << 24)
def _msbStrToInt(str):
	return (ord(str[0]) << 24) + \
	       (ord(str[1]) << 16) + \
	       (ord(str[2]) << 8) + \
	       ord(str[3])
def _intToLsbStr(int):
	return chr(int         & 0xff) + \
	       chr((int >> 8)  & 0xff) + \
	       chr((int >> 16) & 0xff) + \
	       chr((int >> 24) & 0xff)

def _getpos(levels = 0):
	"""Returns the position in the code where the function was called.
	The function uses some knowledge about python stack frames."""
	import sys
	# get access to the stack frame by generating an exception.
	try:
		raise RuntimeError
	except RuntimeError:
		frame = sys.exc_traceback.tb_frame
	frame = frame.f_back # caller's frame
	while levels > 0:
		frame = frame.f_back
		levels = levels - 1
	return (frame.f_globals['__name__'],
		frame.f_code.co_name,
		frame.f_lineno)

class Catalog:
	def __init__(self, domain=None, localedir=localedir):
		self.domain = domain
		self.localedir = localedir
		self.cat = {}
		if not domain: return
		for self.lang in lang:
			if self.lang == 'C':
				return
			catalog = "%s//%s/LC_MESSAGES/%s.mo" % (
				localedir, self.lang, domain)
			try:
				f = open(catalog, "rb")
				buffer = f.read()
				del f
				break
			except IOError:
				pass
		else:
			return # assume C locale

		strToInt = _lsbStrToInt
		if strToInt(buffer[:4]) != 0x950412de:
			# catalog is encoded with MSB offsets.
			strToInt = _msbStrToInt
			if strToInt(buffer[:4]) != 0x950412de:
				# magic number doesn't match
				raise error, 'Bad magic number in %s' % (catalog,)

		self.revision = strToInt(buffer[4:8])
		nstrings = strToInt(buffer[8:12])
		origTabOffset  = strToInt(buffer[12:16])
		transTabOffset = strToInt(buffer[16:20])
		for i in range(nstrings):
			origLength = strToInt(buffer[origTabOffset:
						     origTabOffset+4])
			origOffset = strToInt(buffer[origTabOffset+4:
						     origTabOffset+8])
			origTabOffset = origTabOffset + 8
			origStr = buffer[origOffset:origOffset+origLength]
		
			transLength = strToInt(buffer[transTabOffset:
						      transTabOffset+4])
			transOffset = strToInt(buffer[transTabOffset+4:
						      transTabOffset+8])
			transTabOffset = transTabOffset + 8
			transStr = buffer[transOffset:transOffset+transLength]
			
			self.cat[origStr] = transStr

	def gettext(self, string):
		"""Get the translation of a given string"""
		if self.cat.has_key(string):
			return self.cat[string]
		else:
			return string
	# allow catalog access as cat(str) and cat[str] and cat.gettext(str)
	__getitem__ = gettext
	__call__ = gettext

	# this is experimental code for producing mo files from Catalog objects
	def __setitem__(self, string, trans):
		"""Set the translation of a given string"""
		self.cat[string] = trans
	def save(self, file):
		"""Create a .mo file from a Catalog object"""
		try:
			f = open(file, "wb")
		except IOError:
			raise error, "can't open " + file + " for writing"
		f.write(_intToLsbStr(0x950412de))    # magic number
		f.write(_intToLsbStr(0))             # revision
		f.write(_intToLsbStr(len(self.cat))) # nstrings

		oIndex = []; oData = ''
		tIndex = []; tData = ''
		for orig, trans in self.cat.items():
			oIndex.append((len(orig), len(oData)))
			oData = oData + orig + '\0'
			tIndex.append((len(trans), len(tData)))
			tData = tData + trans + '\0'
		oIndexOfs = 20
		tIndexOfs = oIndexOfs + 8 * len(oIndex)
		oDataOfs = tIndexOfs + 8 * len(tIndex)
		tDataOfs = oDataOfs + len(oData)
		f.write(_intToLsbStr(oIndexOfs))
		f.write(_intToLsbStr(tIndexOfs))
		for length, offset in oIndex:
			f.write(_intToLsbStr(length))
			f.write(_intToLsbStr(offset + oDataOfs))
		for length, offset in tIndex:
			f.write(_intToLsbStr(length))
			f.write(_intToLsbStr(offset + tDataOfs))
		f.write(oData)
		f.write(tData)

_cat = None
_cats = {}

if xgettext:
	class Catalog:
		def __init__(self, domain, localedir):
			self.domain = domain
			self.localedir = localedir
			self._strings = {}
		def gettext(self, string):
			# there is always one level of redirection for calls
			# to this function
			pos = _getpos(2) # get this function's caller
			if self._strings.has_key(string):
				if pos not in self._strings[string]:
					self._strings[string].append(pos)
			else:
				self._strings[string] = [pos]
			return string
		__getitem__ = gettext
		__call__ = gettext
		def __setitem__(self, item, data):
			pass
		def save(self, file):
			pass
		def output(self, fp):
			import string
			fp.write('# POT file for domain %s\n' % (self.domain,))
			for str in self._strings.keys():
				pos = map(lambda x: "%s(%s):%d" % x,
					  self._strings[str])
				pos.sort()
				length = 80
				for p in pos:
					if length + len(p) > 74:
						fp.write('\n#:')
						length = 2
					fp.write(' ')
					fp.write(p)
					length = length + 1 + len(p)
				fp.write('\n')
				if '\n' in str:
					fp.write('msgid ""\n')
					lines = string.split(str, '\n')
					lines = map(lambda x:
						    '"%s\\n"\n' % (x,),
						    lines[:-1]) + \
						    ['"%s"\n' % (lines[-1],)]
					fp.writelines(lines)
				else:
					fp.write('msgid "%s"\n' % (str,))
				fp.write('msgstr ""\n')
				
	import sys
	if hasattr(sys, 'exitfunc'):
		_exitchain = sys.exitfunc
	else:
		_exitchain = None
	def exitfunc(dir=xgettext, _exitchain=_exitchain):
		# actually output all the .pot files.
		import os
		for file in _cats.keys():
			fp = open(os.path.join(dir, file + '.pot'), 'w')
			cat = _cats[file]
			cat.output(fp)
			fp.close()
		if _exitchain: _exitchain()
	sys.exitfunc = exitfunc
	del sys, exitfunc, _exitchain, xgettext

def bindtextdomain(domain, localedir=localedir):
	global _cat
	if not _cats.has_key(domain):
		_cats[domain] = Catalog(domain, localedir)
	if not _cat: _cat = _cats[domain]

def textdomain(domain):
	global _cat
	if not _cats.has_key(domain):
		_cats[domain] = Catalog(domain)
	_cat = _cats[domain]

def gettext(string):
	if _cat == None: raise error, "No catalog loaded"
	return _cat.gettext(string)

_ = gettext

def dgettext(domain, string):
	if domain is None:
		return gettext(string)
	if not _cats.has_key(domain):
		raise error, "Domain '" + domain + "' not loaded"
	return _cats[domain].gettext(string)

def test():
	import sys
	global localedir
	if len(sys.argv) not in (2, 3):
		print "Usage: %s DOMAIN [LOCALEDIR]" % (sys.argv[0],)
		sys.exit(1)
	domain = sys.argv[1]
	if len(sys.argv) == 3:
		bindtextdomain(domain, sys.argv[2])
	textdomain(domain)
	info = gettext('')  # this is where special info is often stored
	if info:
		print "Info for domain %s, lang %s." % (domain, _cat.lang)
		print info
	else:
		print "No info given in mo file."

if __name__ == '__main__':
	test()




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2001  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path
if __name__ == "__main__":
    sys.path.append(".")
import gettext
from i18n import _
import re, string
tokens = (
 'NAME', 'EQUALS', 'STRING', 'I18NSTRING', 'INTEGER', 'LBRACE',
 'RBRACE', 'COMMENT', 'PLUS', 'COMMA', 'PERCENT', 'DIVIDE',
 'BLOCKSTART', 'EQUALS_EXP', 'PLUS_EXP', 'DIVIDE_EXP', 'PERCENT_EXP',
 'COMMA_EXP', 'TEMPO', 'INCLUDE')

for t in tokens:
    globals()[t] = t

re_INCLUDE = re.compile("include*\(([\w-]*)\)")
re_ALL = re.compile(r"""
            (?P<keyword>[a-zA-Z]+[\w_-]*(\(.*?\))?)|
        ###### xstring
          (
           \"\"\"(?P<xstring>(.*?))\"\"\"
           )|
        ###### string
            ((?P<i18npre>_\()?\"
            (?P<string>[^\"]*)
            \"(?P<i18npost>\))?)|
        ###### integer
            (?P<integer>-?\d+)|
        ###### rational
            (?P<rat_num>\d+)/(?P<rat_den>\d+)|
        ###### comment
            (?P<comment>\#.*?$)
            """, re.VERBOSE|re.MULTILINE|re.DOTALL)

class DataparserException:
    def __init__(self, filename, lineno, badcode):
        self.filename = filename
        self.lineno = lineno
        self.badcode = badcode
    def __str__(self):
        return "dataparser-error, lineno: %i\nThis code is invalid: %s: %s" % (self.filename, self.lineno, self.badcode)


class Dataparser:
    def __init__(self, globals={}, gd=[]):
        self.m_globals = globals
        self.m_gd = gd
    def parse_file(self, filename, only_header=0):
        self.m_filename = [filename]
        v = []
        self.m_blocks = []
        context = self.m_globals
        v = self.get_tokens(filename, only_header)
        v = self.do_include(v)
        for op in (PERCENT, PLUS, DIVIDE, COMMA, EQUALS):
            v = self.do_simple_op(v, op)
        v = self.do_blockstart(v)
        for ex in v:
            if ex[0] == EQUALS_EXP:
                context[ex[1][1]] = self.evaluate(ex[2])
            elif ex[0] == BLOCKSTART:
                assert context == self.m_globals
                self.m_blocks.append({})
                context = self.m_blocks[-1]
                context['blocktype'] = ex[1][1]
            elif ex[0] == RBRACE:
                for n in self.m_gd:
                    if not context.has_key(n):
                        context[n] = self.m_globals[n]
                context = self.m_globals
            elif ex[0] == STRING:
                # the shortcut that interprets a single string
                # "this is a string" as
                # music = "this is a string"
                context['music'] = ex[1]
            elif ex[0] == PERCENT_EXP:
                # the shortcut that interprets a single string
                # "this is a string" as
                # music = "this is a string"
                context['music'] = self.evaluate(ex)
            elif ex[0] == PLUS_EXP:
                # the shortcut that interprets a single string
                # "this is a string" as
                # music = "this is a string"
                context['music'] = self.evaluate(ex)
            else:
                raise "Parse error in '%s', line %i" % (ex[2], ex[3]), ex[1]
    def get_tokens(self, filename, first_block_only):
        f = open(filename, "r")
        s = f.read()
        f.close()
        linecount = 1
        p = 0
        v = []
        slen = len(s)
        while p < slen:
            # doing simple comparations is faster than using a
            # regular expression
            if s[p] in " \n\t{=}%+,/":
                if s[p] == ' ':
                    p += 1
                    continue
                if s[p] == '\n':
                    p += 1
                    linecount += 1
                    continue
                if s[p] == '\t':
                    p += 1
                    continue
                if s[p] == '{':
                    p += 1
                    v.append((LBRACE, '{', filename, linecount))
                    continue
                elif s[p] == '=':
                    p += 1
                    v.append((EQUALS, '=', filename, linecount))
                    continue
                elif s[p] == '}':
                    p += 1
                    v.append((RBRACE, '}', filename, linecount))
                    if first_block_only:
                        break
                    continue
                elif s[p] == '%':
                    p += 1
                    v.append((PERCENT, '%', filename, linecount))
                    continue
                elif s[p] == '+':
                    p += 1
                    v.append((PLUS, '+', filename, linecount))
                    continue
                elif s[p] == ',':
                    p += 1
                    v.append((COMMA, ',', filename, linecount))
                    continue
                elif s[p] == '/':
                    p += 1
                    v.append((DIVIDE, '/', filename, linecount))
                    continue
            m = re_ALL.match(s[p:])
            if m:
                if m.group('integer'):
                    p += m.end()
                    v.append((INTEGER, int(m.group('integer')),
                              filename, linecount))
                    continue
                if m.group('string'):
                    p += m.end()
                    linecount = linecount + string.count(m.group(), '\n')
                    if m.group('i18npre'):
                        v.append((I18NSTRING, m.group('string'),
                                  filename, linecount))
                    else:
                        v.append((STRING, m.group('string'),
                                  filename, linecount))
                    continue
                elif m.group('xstring'):
                    #FIXME can't xstring and string be joined in the regex?
                    p += m.end()
                    linecount += string.count(m.group(), '\n')
                    v.append((STRING, m.group('xstring'), filename, linecount))
                    continue
                elif m.group('keyword'):
                    p += m.end()
                    im = re_INCLUDE.match(m.group('keyword'))
                    if im:
                        v.append((INCLUDE, im.groups()[0], filename, linecount))
                    else:
                        v.append((NAME, m.group('keyword'), filename, linecount))
                    continue
                elif m.group('comment'):
                    p += m.end()
                    continue
                elif m.group('rat_num'):
                    p += m.end()
                    v.append((TEMPO, (int(m.group('rat_num')),
                                      int(m.group('rat_den')))))
                    continue
                else:
                    raise "unmatched", m.groups()
            else:
                raise "miss '%s'" % s[p:p+10]
        return v
    def do_include(self, list):
        x = 0
        while x < len(list):
            if list[x][0] == INCLUDE:
                fn = os.path.join(os.path.dirname(self.m_filename[-1]),
                                  list[x][1])
                list[x:x+1] = self.get_tokens(fn, 0)
            x = x + 1
        return list
    def do_simple_op(self, list, op):
        x = 1
        s = '%s_EXP' % op
        while x + 1 < len(list):
            if list[x][0] is op:
                list[x-1:x+2]=[(s, list[x-1], list[x+1])]
            else:
                x += 1
        return list
    def do_blockstart(self, list):
        x = 0
        while x < len(list):
            if list[x][0] is NAME and list[x+1][0] is LBRACE:
                list[x:x+2]=[('BLOCKSTART', list[x])]
            else:
                x += 1
        return list
    def evaluate(self, exp):
        if exp[0] == DIVIDE_EXP:
            return int(exp[1][1]), int(exp[2][1])
        elif exp[0] == STRING:
            return exp[1]
        elif exp[0] == I18NSTRING:
            return _(exp[1])
        elif exp[0] == INTEGER:
            return exp[1]
        elif exp[0] == PLUS_EXP:
            return self.evaluate(exp[1]) + self.evaluate(exp[2])
        elif exp[0] == PERCENT_EXP:
            if exp[1][0] is NAME:
                A = self.evaluate(exp[1])
            else:
                A = exp[1][1]
            return A % exp[2][1]
        elif exp[0] == COMMA_EXP:
            A = self.evaluate(exp[1])
            B = self.evaluate(exp[2])
            if type(A) == type([]):
                return A + [B]
            else:
                return [A, B]
        elif exp[0] == NAME:
            return self.m_globals[exp[1]]
        elif exp[0] == TEMPO:
            return exp[1]
        else:
            raise "unfinished", exp

def get_translated_string(dict, name):
    for n in gettext.lang:
        if dict.has_key("%s(%s)" % (name, n)):
            return dict["%s(%s)" % (name, n)]
    return dict[name]

def __f():
    if len(sys.argv) == 1:
        print "Give the file to parse as command line argument."
        sys.exit(-1)
    import time
    t1 = time.clock()
    for fn in sys.argv[1:]:
        if (not os.path.isfile(fn)) or (fn == 'lesson-files/Makefile'):
            continue
        p = Dataparser({'dictation': 'dictation',
                      'progression': 'progression',
                      'harmony': 'harmony',
                      'sing-chord': 'sing-chord',
                      'chord': 'chord',
                      'id-by-name': 'id-by-name',
                      'satb': 'satb',
                      'horiz': 'horiz',
                      'vertic': 'vertic',
                      'yes': 1,
                      'no': 0,
                      'tempo': (160, 4)})
        for x in range(10):
            p.parse_file(fn)
       #print p.m_blocks
    print time.clock() - t1

def _benchmark_tokenizer():
    import time
    n = 40
    p = Dataparser()
    t1 = time.clock()
    for fn in sys.argv[1:]:
        if os.path.isfile(fn) and (os.path.basename(fn) != 'Makefile'):
            for x in range(n):
                v = p.get_tokens(fn, 0)
    t2 = time.clock()
    return t2-t1

if __name__ == "__main__":
    import i18n
    i18n.initialise("share/locale")
    from i18n import _
    #print _benchmark_tokenizer()
    __f()
    #import profile, pstats
    #profile.run("__f()", "profile.txt")
    #s = pstats.Stats("profile.txt")
    #s.strip_dirs().sort_stats('cumulative').print_stats(100)
